@@ -37,6 +37,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
+#include <gtk/gtkx.h>
 
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
@@ -45,15 +46,16 @@
 #include <libxfce4kbd-private/xfce-shortcuts-provider.h>
 #include <libxfce4kbd-private/xfce-shortcuts-xfwm4.h>
 
+#include <common/xfwm-common.h>
+
 #include "xfwm4-dialog_ui.h"
 #include "xfwm4-settings.h"
+#include "range-debouncer.h"
 
 
 
 #define DEFAULT_THEME                   "Default"
 
-#define INDICATOR_SIZE                  9
-
 #define SHORTCUTS_NAME_COLUMN           0
 #define SHORTCUTS_FEATURE_COLUMN        1
 #define SHORTCUTS_SHORTCUT_COLUMN       2
@@ -80,8 +82,10 @@ enum
 
 
 
-static void       xfwm_settings_class_init                           (XfwmSettingsClass    *klass);
-static void       xfwm_settings_init                                 (XfwmSettings         *settings);
+static void       xfwm_settings_class_init                           (XfwmSettingsClass    *klass,
+                                                                      gpointer              data);
+static void       xfwm_settings_init                                 (XfwmSettings         *settings,
+                                                                      gpointer              data);
 static void       xfwm_settings_constructed                          (GObject              *object);
 static void       xfwm_settings_finalize                             (GObject              *object);
 static void       xfwm_settings_get_property                         (GObject              *object,
@@ -94,7 +98,8 @@ static void       xfwm_settings_set_property                         (GObject
                                                                       GParamSpec           *pspec);
 static gint       xfwm_settings_theme_sort_func                      (GtkTreeModel         *model,
                                                                       GtkTreeIter          *iter1,
-                                                                      GtkTreeIter          *iter2);
+                                                                      GtkTreeIter          *iter2,
+                                                                      gpointer              data);
 static void       xfwm_settings_load_themes                          (XfwmSettings         *settings);
 static void       xfwm_settings_theme_selection_changed              (GtkTreeSelection     *selection,
                                                                       XfwmSettings         *settings);
@@ -126,12 +131,7 @@ static void       xfwm_settings_hidden_frame_drag_data               (GtkWidget
                                                                       guint                 info,
                                                                       guint                 timestamp,
                                                                       XfwmSettings         *settings);
-static void       xfwm_settings_delete_indicator                     (GtkWidget            *widget);
-static void       xfwm_settings_create_indicator                     (GtkWidget            *widget,
-                                                                      gint                  x,
-                                                                      gint                  y,
-                                                                      gint                  width,
-                                                                      gint                  height);
+static gboolean   xfwm_settings_title_button_press_event             (GtkWidget            *widget);
 static void       xfwm_settings_title_button_drag_data               (GtkWidget            *widget,
                                                                       GdkDragContext       *drag_context,
                                                                       GtkSelectionData     *data,
@@ -141,6 +141,9 @@ static void       xfwm_settings_title_button_drag_begin              (GtkWidget
                                                                       GdkDragContext       *drag_context);
 static void       xfwm_settings_title_button_drag_end                (GtkWidget            *widget,
                                                                       GdkDragContext       *drag_context);
+static gboolean   xfwm_settings_active_frame_draw                    (GtkWidget            *widget,
+                                                                      cairo_t              *cr,
+                                                                      XfwmSettings         *settings);
 static gboolean   xfwm_settings_signal_blocker                       (GtkWidget            *widget);
 static GdkPixbuf *xfwm_settings_create_icon_from_widget              (GtkWidget            *widget);
 
@@ -235,7 +238,7 @@ static const MenuTemplate title_align_values[] = {
 
 
 static gboolean           opt_version = FALSE;
-static GdkNativeWindow    opt_socket_id = 0;
+static Window             opt_socket_id = 0;
 static GOptionEntry       opt_entries[] = {
   { "socket-id", 's', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_INT, &opt_socket_id,
     N_("Settings manager socket"), N_("SOCKET ID") },
@@ -275,7 +278,7 @@ xfwm_settings_get_type (void)
 
 
 static void
-xfwm_settings_class_init (XfwmSettingsClass *klass)
+xfwm_settings_class_init (XfwmSettingsClass *klass, gpointer data)
 {
   GObjectClass *gobject_class;
 
@@ -303,7 +306,7 @@ xfwm_settings_class_init (XfwmSettingsClass *klass)
 
 
 static void
-xfwm_settings_init (XfwmSettings *settings)
+xfwm_settings_init (XfwmSettings *settings, gpointer data)
 {
   settings->priv = XFWM_SETTINGS_GET_PRIVATE (settings);
 
@@ -443,6 +446,10 @@ xfwm_settings_constructed (GObject *object)
     g_signal_connect (hidden_frame, "drag-data-received",
                       G_CALLBACK (xfwm_settings_hidden_frame_drag_data), settings);
 
+    g_object_set_data (G_OBJECT (active_frame), "indicator-position", GINT_TO_POINTER (-1));
+    g_signal_connect (active_frame, "draw",
+                      G_CALLBACK (xfwm_settings_active_frame_draw), settings);
+
     children = gtk_container_get_children (GTK_CONTAINER (active_box));
     for (list_iter = children; list_iter != NULL; list_iter = g_list_next (list_iter))
       {
@@ -467,7 +474,7 @@ xfwm_settings_constructed (GObject *object)
         g_signal_connect (button, "drag_end", G_CALLBACK (xfwm_settings_title_button_drag_end),
                           NULL);
         g_signal_connect (button, "button_press_event",
-                          G_CALLBACK (xfwm_settings_signal_blocker), NULL);
+                          G_CALLBACK (xfwm_settings_title_button_press_event), NULL);
         g_signal_connect (button, "enter_notify_event",
                           G_CALLBACK (xfwm_settings_signal_blocker), NULL);
         g_signal_connect (button, "focus",  G_CALLBACK (xfwm_settings_signal_blocker), NULL);
@@ -490,7 +497,7 @@ xfwm_settings_constructed (GObject *object)
           g_signal_connect (button, "drag_end", G_CALLBACK (xfwm_settings_title_button_drag_end),
                             NULL);
           g_signal_connect (button, "button_press_event",
-                            G_CALLBACK (xfwm_settings_signal_blocker), NULL);
+                            G_CALLBACK (xfwm_settings_title_button_press_event), NULL);
           g_signal_connect (button, "enter_notify_event",
                             G_CALLBACK (xfwm_settings_signal_blocker), NULL);
           g_signal_connect (button, "focus",  G_CALLBACK (xfwm_settings_signal_blocker), NULL);
@@ -514,8 +521,8 @@ xfwm_settings_constructed (GObject *object)
 
   /* Set reset button icon */
   gtk_button_set_image (GTK_BUTTON (shortcuts_reset_button),
-                        gtk_image_new_from_stock (GTK_STOCK_REVERT_TO_SAVED,
-                                                  GTK_ICON_SIZE_BUTTON));
+                        gtk_image_new_from_icon_name ("document-revert",
+                                                      GTK_ICON_SIZE_BUTTON));
 
   /* Keyboard tab: Shortcuts tree view */
   {
@@ -558,9 +565,9 @@ xfwm_settings_constructed (GObject *object)
 
   /* Focus tab */
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/focus_delay", G_TYPE_INT,
-                          gtk_range_get_adjustment (GTK_RANGE (focus_delay_scale)), "value");
+                          range_debouncer_bind (GTK_RANGE (focus_delay_scale)), "value");
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/raise_delay", G_TYPE_INT,
-                          gtk_range_get_adjustment (GTK_RANGE (focus_raise_delay_scale)), "value");
+                          range_debouncer_bind (GTK_RANGE (focus_raise_delay_scale)), "value");
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/raise_on_click", G_TYPE_BOOLEAN,
                           raise_on_click_check, "active");
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/raise_on_focus", G_TYPE_BOOLEAN,
@@ -627,9 +634,9 @@ xfwm_settings_constructed (GObject *object)
 
   /* Advanced tab */
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/snap_width", G_TYPE_INT,
-                          gtk_range_get_adjustment (GTK_RANGE (snap_width_scale)), "value");
+                          range_debouncer_bind (GTK_RANGE (snap_width_scale)), "value");
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/wrap_resistance", G_TYPE_INT,
-                          gtk_range_get_adjustment (GTK_RANGE (wrap_resistance_scale)), "value");
+                          range_debouncer_bind (GTK_RANGE (wrap_resistance_scale)), "value");
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/box_move", G_TYPE_BOOLEAN,
                           box_move_check, "active");
   xfconf_g_property_bind (settings->priv->wm_channel, "/general/box_resize", G_TYPE_BOOLEAN,
@@ -735,7 +742,8 @@ xfwm_settings_new (void)
 static gint
 xfwm_settings_theme_sort_func (GtkTreeModel *model,
                                GtkTreeIter  *iter1,
-                               GtkTreeIter  *iter2)
+                               GtkTreeIter  *iter2,
+                               gpointer      data)
 {
   gchar *str1 = NULL;
   gchar *str2 = NULL;
@@ -842,7 +850,7 @@ xfwm_settings_create_dialog (XfwmSettings *settings)
 
 static GtkWidget *
 xfwm_settings_create_plug (XfwmSettings   *settings,
-                           GdkNativeWindow socket_id)
+                           Window          socket_id)
 {
   GtkWidget *plug;
   GtkWidget *child;
@@ -853,7 +861,7 @@ xfwm_settings_create_plug (XfwmSettings   *settings,
   gtk_widget_show (plug);
 
   child = GTK_WIDGET (gtk_builder_get_object (settings->priv->builder, "plug-child"));
-  gtk_widget_reparent (child, plug);
+  xfwm_widget_reparent (child, plug);
   gtk_widget_show (child);
 
   return plug;
@@ -944,7 +952,7 @@ main (int    argc,
       g_signal_connect (dialog, "response", G_CALLBACK (xfwm_settings_response), NULL);
 
       /* To prevent the settings dialog to be saved in the session */
-      gdk_set_sm_client_id ("FAKE ID");
+      gdk_x11_set_sm_client_id ("FAKE ID");
 
       gtk_main ();
 
@@ -956,7 +964,7 @@ main (int    argc,
       g_signal_connect (plug, "delete-event", G_CALLBACK (gtk_main_quit), NULL);
 
       /* To prevent the settings dialog to be saved in the session */
-      gdk_set_sm_client_id ("FAKE ID");
+      gdk_x11_set_sm_client_id ("FAKE ID");
 
       /* Stop startup notification */
       gdk_notify_startup_complete ();
@@ -1049,13 +1057,14 @@ xfwm_settings_active_frame_drag_data (GtkWidget        *widget,
                                       guint             timestamp,
                                       XfwmSettings     *settings)
 {
-  GtkWidget *source;
-  GtkWidget *parent;
-  GtkWidget *active_box;
-  GList     *children;
-  GList     *iter;
-  gint       xoffset;
-  gint       i;
+  GtkWidget     *source;
+  GtkWidget     *parent;
+  GtkWidget     *active_box;
+  GList         *children;
+  GList         *iter;
+  GtkAllocation  allocation;
+  gint           xoffset;
+  gint           i;
 
   g_return_if_fail (XFWM_IS_SETTINGS (settings));
 
@@ -1070,17 +1079,20 @@ xfwm_settings_active_frame_drag_data (GtkWidget        *widget,
   gtk_box_pack_start (GTK_BOX (active_box), source, info == 3, info == 3, 0);
   g_object_unref (source);
 
-  xoffset = widget->allocation.x;
+  gtk_widget_get_allocation (widget, &allocation);
+
+  xoffset = allocation.x;
 
   children = gtk_container_get_children (GTK_CONTAINER (active_box));
 
   for (i = 0, iter = children; iter != NULL; ++i, iter = g_list_next (iter))
-    if (GTK_WIDGET_VISIBLE (iter->data))
-      if (x < (GTK_WIDGET (iter->data)->allocation.width / 2 +
-               GTK_WIDGET (iter->data)->allocation.x - xoffset))
-        {
+    if (gtk_widget_get_visible (GTK_WIDGET (iter->data)))
+      {
+        gtk_widget_get_allocation (GTK_WIDGET (iter->data), &allocation);
+
+        if (x < allocation.width / 2 + allocation.x - xoffset)
           break;
-        }
+      }
 
   g_list_free (children);
 
@@ -1099,53 +1111,40 @@ xfwm_settings_active_frame_drag_motion (GtkWidget      *widget,
                                         guint           timestamp,
                                         XfwmSettings   *settings)
 {
-  GtkWidget *active_box;
-  GdkWindow *indicator;
-  GList     *children;
-  GList     *iter;
-  gint       xoffset = widget->allocation.x;
-  gint       height;
-  gint       ix;
-  gint       iy;
+  GtkWidget     *active_box;
+  GList         *children;
+  GList         *iter;
+  GtkAllocation  allocation;
+  gint           xoffset;
+  gint           index = -1;
 
   g_return_val_if_fail (XFWM_IS_SETTINGS (settings), FALSE);
 
+  gtk_widget_get_allocation (widget, &allocation);
+
+  xoffset = allocation.x;
+
   active_box = GTK_WIDGET (gtk_builder_get_object (settings->priv->builder, "active-box"));
   children = gtk_container_get_children (GTK_CONTAINER (active_box));
 
   /* Set a value so that the compiler does not (rightfully) complain */
-  ix = INDICATOR_SIZE;
-  for (iter = children; iter != NULL; iter = g_list_next (iter))
+  for (iter = children, index = 0; iter != NULL; iter = g_list_next (iter))
     {
-      if (GTK_WIDGET_VISIBLE (iter->data))
+      if (gtk_widget_get_visible (GTK_WIDGET (iter->data)))
         {
-          if (x < (GTK_WIDGET (iter->data)->allocation.width / 2 +
-                   GTK_WIDGET (iter->data)->allocation.x - xoffset))
-            {
-              ix = GTK_WIDGET (iter->data)->allocation.x;
-              break;
-            }
+          gtk_widget_get_allocation (GTK_WIDGET (iter->data), &allocation);
 
-          ix = GTK_WIDGET (iter->data)->allocation.x + GTK_WIDGET (iter->data)->allocation.width;
+          if (x < (allocation.width / 2 + allocation.x - xoffset))
+            break;
+
+          index++;
         }
     }
 
   g_list_free (children);
 
-  ix -= INDICATOR_SIZE / 2 + 1;
-  iy = active_box->allocation.y - INDICATOR_SIZE / 2 +
-       gtk_container_get_border_width (GTK_CONTAINER (active_box));
-
-  indicator = g_object_get_data (G_OBJECT (active_box), "indicator_window");
-
-  if (G_UNLIKELY (indicator == NULL))
-    {
-      height = active_box->allocation.height + INDICATOR_SIZE -
-               gtk_container_get_border_width (GTK_CONTAINER (active_box)) * 2;
-      xfwm_settings_create_indicator (active_box, ix, iy, INDICATOR_SIZE, height);
-    }
-  else
-    gdk_window_move (indicator, ix, iy);
+  g_object_set_data (G_OBJECT (widget), "indicator-position", GINT_TO_POINTER (index));
+  gtk_widget_queue_draw (widget);
 
   return FALSE;
 }
@@ -1160,7 +1159,8 @@ xfwm_settings_active_frame_drag_leave (GtkWidget      *widget,
 {
   g_return_if_fail (XFWM_IS_SETTINGS (settings));
 
-  xfwm_settings_delete_indicator (GTK_WIDGET (gtk_builder_get_object (settings->priv->builder, "active-box")));
+  g_object_set_data (G_OBJECT (widget), "indicator-position", GINT_TO_POINTER (-1));
+  gtk_widget_queue_draw (widget);
 }
 
 
@@ -1202,82 +1202,19 @@ xfwm_settings_hidden_frame_drag_data (GtkWidget        *widget,
 
 
 
-static void
-xfwm_settings_create_indicator (GtkWidget *widget,
-                                gint       x,
-                                gint       y,
-                                gint       width,
-                                gint       height)
+static gboolean
+xfwm_settings_title_button_press_event (GtkWidget *widget)
 {
-  GdkWindowAttr attributes;
-  GdkWindow    *indicator;
-  GdkRegion    *shape;
-  GdkPoint      points[9];
-  gint          attr_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_COLORMAP | GDK_WA_VISUAL;
+  GdkPixbuf *pixbuf;
 
-  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), TRUE);
 
-  attributes.title = NULL;
-  attributes.event_mask = 0;
-  attributes.x = x;
-  attributes.y = y;
-  attributes.width = width;
-  attributes.height = height;
-  attributes.wclass = GDK_INPUT_OUTPUT;
-  attributes.visual = gtk_widget_get_visual (widget);
-  attributes.colormap = gtk_widget_get_colormap (widget);
-  attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.cursor = NULL;
-  attributes.wmclass_name = NULL;
-  attributes.wmclass_class = NULL;
-  attributes.override_redirect = FALSE;
+  /* set pixbuf before drag begin cause it can be not displayed */
+  pixbuf = xfwm_settings_create_icon_from_widget (widget);
+  gtk_drag_source_set_icon_pixbuf (widget, pixbuf);
+  g_object_unref (pixbuf);
 
-  indicator = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attr_mask);
-  gdk_window_set_user_data (indicator, widget);
-  g_object_set_data (G_OBJECT (widget), "indicator_window", indicator);
-
-  points[0].x = 0;
-  points[0].y = 0;
-  points[1].x = width;
-  points[1].y = 0;
-  points[2].x = width / 2 + 1;
-  points[2].y = width / 2;
-  points[3].x = width / 2 + 1;
-  points[3].y = height - 1 - width / 2;
-  points[4].x = width;
-  points[4].y = height;
-  points[5].x = 0;
-  points[5].y = height - 1;
-  points[6].x = width / 2;
-  points[6].y = height - 1 - width / 2;
-  points[7].x = width / 2;
-  points[7].y = width / 2;
-  points[8].x = 0;
-  points[8].y = 0;
-
-  shape = gdk_region_polygon (points, 9, GDK_WINDING_RULE);
-  gdk_window_shape_combine_region (indicator, shape, 0, 0);
-
-  gdk_window_show (indicator);
-  gdk_window_raise (indicator);
-}
-
-
-
-static void
-xfwm_settings_delete_indicator (GtkWidget *widget)
-{
-  GdkWindow *indicator;
-
-  g_return_if_fail (GTK_IS_WIDGET (widget));
-
-  indicator = g_object_get_data (G_OBJECT (widget), "indicator_window");
-
-  if (G_LIKELY (indicator != NULL))
-    {
-      gdk_window_destroy (indicator);
-      g_object_set_data (G_OBJECT (widget), "indicator_window", NULL);
-    }
+  return TRUE;
 }
 
 
@@ -1304,14 +1241,8 @@ static void
 xfwm_settings_title_button_drag_begin (GtkWidget      *widget,
                                        GdkDragContext *drag_context)
 {
-  GdkPixbuf *pixbuf;
-
   g_return_if_fail (GTK_IS_WIDGET (widget));
 
-  pixbuf = xfwm_settings_create_icon_from_widget (widget);
-  gtk_drag_source_set_icon_pixbuf (widget, pixbuf);
-  g_object_unref (pixbuf);
-
   gtk_widget_hide (widget);
 }
 
@@ -1326,6 +1257,83 @@ xfwm_settings_title_button_drag_end (GtkWidget      *widget,
 
 
 
+static gboolean
+xfwm_settings_active_frame_draw (GtkWidget    *widget,
+                                 cairo_t      *cr,
+                                 XfwmSettings *settings)
+{
+  GtkWidget     *active_box;
+  gint           position;
+  GList         *children;
+  GList         *iter;
+  GtkAllocation  widget_allocation;
+  GtkAllocation  box_allocation;
+  gint           index;
+  gint           x;
+  gint           spacing;
+  gint           hshift;
+  GdkRGBA        color;
+
+  active_box = GTK_WIDGET (gtk_builder_get_object (settings->priv->builder, "active-box"));
+
+  position = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "indicator-position"));
+
+  if (position >= 0)
+    {
+      gtk_widget_get_allocation (active_box, &box_allocation);
+
+      spacing = gtk_box_get_spacing (GTK_BOX (active_box));
+      x = box_allocation.x - spacing;
+
+      children = gtk_container_get_children (GTK_CONTAINER (active_box));
+
+      for (iter = children, index = 1; iter != NULL; iter = g_list_next (iter))
+        {
+          if (gtk_widget_get_visible (GTK_WIDGET (iter->data)))
+            {
+              if (index == position)
+                {
+                  gtk_widget_get_allocation (GTK_WIDGET (iter->data), &widget_allocation);
+                  x = widget_allocation.x + widget_allocation.width;
+                }
+
+              index++;
+            }
+        }
+
+      g_list_free (children);
+
+      gtk_widget_get_allocation (widget, &widget_allocation);
+
+      gtk_style_context_get_color (gtk_widget_get_style_context (widget),
+                                   GTK_STATE_FLAG_NORMAL, &color);
+
+      cairo_translate (cr,
+                       box_allocation.x - widget_allocation.x,
+                       box_allocation.y - widget_allocation.y);
+
+      hshift = spacing * 5 / 3;
+      x -= box_allocation.x + (hshift - spacing) / 2;
+
+      gdk_cairo_set_source_rgba (cr, &color);
+      cairo_move_to (cr, x, -spacing);
+      cairo_rel_line_to (cr, hshift, 0);
+      cairo_rel_line_to (cr, -(hshift / 2 - 1), spacing);
+      cairo_rel_line_to (cr, 0, box_allocation.height);
+      cairo_rel_line_to (cr, hshift / 2 - 1, spacing);
+      cairo_rel_line_to (cr, -hshift, 0);
+      cairo_rel_line_to (cr, hshift / 2 - 1, -spacing);
+      cairo_rel_line_to (cr, 0, -box_allocation.height);
+      cairo_rel_line_to (cr, -(hshift / 2 - 1), -spacing);
+      cairo_close_path (cr);
+      cairo_fill (cr);
+    }
+
+  return FALSE;
+}
+
+
+
 static gboolean
 xfwm_settings_signal_blocker (GtkWidget *widget)
 {
@@ -1337,14 +1345,16 @@ xfwm_settings_signal_blocker (GtkWidget *widget)
 static GdkPixbuf *
 xfwm_settings_create_icon_from_widget (GtkWidget *widget)
 {
-  GdkWindow *drawable;
+  GdkWindow     *window;
+  GtkAllocation  allocation;
 
   g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
-  drawable = GDK_DRAWABLE (gtk_widget_get_parent_window (widget));
-  return gdk_pixbuf_get_from_drawable (NULL, drawable, NULL,
-                                       widget->allocation.x, widget->allocation.y, 0, 0,
-                                       widget->allocation.width, widget->allocation.height);
+  gtk_widget_get_allocation (widget, &allocation);
+  window = gtk_widget_get_parent_window (widget);
+  return gdk_pixbuf_get_from_window (window,
+                                     allocation.x, allocation.y,
+                                     allocation.width, allocation.height);
 }
 
 
@@ -1520,6 +1530,8 @@ xfwm_settings_title_button_alignment_changed (GtkComboBox *combo,
   GtkTreeIter   iter;
   gchar        *value;
   float         align = 0.5f;
+  GList        *children;
+  GList        *citer;
 
   model = gtk_combo_box_get_model (combo);
   if (gtk_combo_box_get_active_iter (combo, &iter))
@@ -1534,7 +1546,16 @@ xfwm_settings_title_button_alignment_changed (GtkComboBox *combo,
       g_free (value);
     }
 
-  gtk_button_set_alignment (GTK_BUTTON (button), align, 0.5f);
+  children = gtk_container_get_children (GTK_CONTAINER (button));
+  for (citer = children; citer != NULL; citer = g_list_next (citer))
+    {
+      if (GTK_IS_LABEL (citer->data))
+        {
+          gtk_label_set_xalign (GTK_LABEL (citer->data), align);
+          break;
+        }
+    }
+  g_list_free (children);
 }
 
 
@@ -1772,6 +1793,22 @@ xfwm_settings_shortcut_removed (XfceShortcutsProvider *provider,
 
 
 
+static void
+free_row (GtkTreeRowReference *reference, gpointer unused)
+{
+  gtk_tree_row_reference_free (reference);
+}
+
+
+
+static void
+free_path (GtkTreePath *path, gpointer unused)
+{
+  gtk_tree_path_free (path);
+}
+
+
+
 static void
 xfwm_settings_shortcut_edit_clicked (GtkButton    *button,
                                      XfwmSettings *settings)
@@ -1811,11 +1848,11 @@ xfwm_settings_shortcut_edit_clicked (GtkButton    *button,
     }
 
   /* Free row reference list */
-  g_list_foreach (row_references, (GFunc) gtk_tree_row_reference_free, NULL);
+  g_list_foreach (row_references, (GFunc) free_row, NULL);
   g_list_free (row_references);
 
   /* Free row list */
-  g_list_foreach (rows, (GFunc) gtk_tree_path_free, NULL);
+  g_list_foreach (rows, (GFunc) free_path, NULL);
   g_list_free (rows);
 }
 
@@ -1879,11 +1916,11 @@ xfwm_settings_shortcut_clear_clicked (GtkButton    *button,
     }
 
   /* Free row reference list */
-  g_list_foreach (row_references, (GFunc) gtk_tree_row_reference_free, NULL);
+  g_list_foreach (row_references, (GFunc) free_row, NULL);
   g_list_free (row_references);
 
   /* Free row list */
-  g_list_foreach (rows, (GFunc) gtk_tree_path_free, NULL);
+  g_list_foreach (rows, (GFunc) free_path, NULL);
   g_list_free (rows);
 }
 
@@ -1893,20 +1930,17 @@ static void
 xfwm_settings_shortcut_reset_clicked (GtkButton    *button,
                                       XfwmSettings *settings)
 {
-  gint response;
+  gint confirm;
 
   g_return_if_fail (XFWM_IS_SETTINGS (settings));
   g_return_if_fail (XFCE_IS_SHORTCUTS_PROVIDER (settings->priv->provider));
 
-  response = xfce_message_dialog (NULL, _("Reset to Defaults"), GTK_STOCK_DIALOG_QUESTION,
-                                  _("Reset to Defaults"),
+  confirm = xfce_dialog_confirm (NULL, "gtk-yes", NULL,
                                   _("This will reset all shortcuts to their default "
                                     "values. Do you really want to do this?"),
-                                  GTK_STOCK_NO, GTK_RESPONSE_NO,
-                                  GTK_STOCK_YES, GTK_RESPONSE_YES,
-                                  NULL);
+                                  _("Reset to Defaults"));
 
-  if (G_LIKELY (response == GTK_RESPONSE_YES))
+  if (confirm)
     xfce_shortcuts_provider_reset_to_defaults (settings->priv->provider);
 }
 
