@@ -164,7 +164,7 @@
 #define CLIENT_FLAG_NAME_CHANGED        (1L<<16)
 #define CLIENT_FLAG_DEMANDS_ATTENTION   (1L<<17)
 #define CLIENT_FLAG_HAS_SHAPE           (1L<<18)
-#define CLIENT_FLAG_FULLSCREN_MONITORS  (1L<<19)
+#define CLIENT_FLAG_FULLSCREEN_MONITORS (1L<<19)
 #define CLIENT_FLAG_HAS_FRAME_EXTENTS   (1L<<20)
 #define CLIENT_FLAG_HIDE_TITLEBAR       (1L<<21)
 #define CLIENT_FLAG_XSYNC_WAITING       (1L<<22)
@@ -469,6 +469,11 @@ void                     clientUpdateMaximizeSize               (Client *);
 gboolean                 clientToggleMaximized                  (Client *,
                                                                  int,
                                                                  gboolean);
+gboolean                 clientToggleMaximizedAtPoint           (Client *,
+                                                                 gint,
+                                                                 gint,
+                                                                 int,
+                                                                 gboolean);
 gboolean                 clientTile                             (Client *,
                                                                  gint,
                                                                  gint,
@@ -489,7 +494,7 @@ void                     clientScreenResize                     (ScreenInfo *,
                                                                  gboolean);
 void                     clientButtonPress                      (Client *,
                                                                  Window,
-                                                                 XButtonEvent *);
+                                                                 XfwmEventButton *);
 xfwmPixmap *             clientGetButtonPixmap                  (Client *,
                                                                  int,
                                                                  int);
