@@ -29,6 +29,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
+#include <X11/XKBlib.h>
 #include <X11/extensions/shape.h>
 
 #include <glib.h>
@@ -69,7 +70,7 @@ clientGetCycleRange (ScreenInfo *screen_info)
     guint range;
 
     g_return_val_if_fail (screen_info != NULL, 0);
-    TRACE ("entering clientGetCycleRange");
+    TRACE ("entering");
 
     range = 0;
     if (screen_info->params->cycle_hidden)
@@ -97,7 +98,7 @@ clientCycleCreateList (Client *c)
     GList *client_list;
 
     g_return_val_if_fail (c, NULL);
-    TRACE ("entering clientCycleCreateList");
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     screen_info = c->screen_info;
     range = clientGetCycleRange (screen_info);
@@ -157,7 +158,7 @@ clientCycleCreateList (Client *c)
             }
         }
 
-        TRACE ("clientCycleCreateList: adding %s", c2->name);
+        TRACE ("adding %s", c2->name);
         client_list = g_list_append (client_list, c2);
     }
 
@@ -172,7 +173,7 @@ clientCycleFocusAndRaise (Client *c)
     Client *sibling;
 
     g_return_if_fail (c != NULL);
-    TRACE ("entering clientFocusAndRaise");
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
@@ -226,6 +227,10 @@ clientCycleUpdateWireframe (Client *c, ClientCycleData *passdata)
 {
     if (c)
     {
+        if (c->screen_info->params->cycle_raise)
+        {
+          clientRaise (c, None);
+        }
         if (passdata->wireframe)
         {
             wireframeUpdate (c, passdata->wireframe);
@@ -236,7 +241,7 @@ clientCycleUpdateWireframe (Client *c, ClientCycleData *passdata)
 }
 
 static eventFilterStatus
-clientCycleEventFilter (XEvent * xevent, gpointer data)
+clientCycleEventFilter (XfwmEvent *event, gpointer data)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -249,7 +254,7 @@ clientCycleEventFilter (XEvent * xevent, gpointer data)
     gboolean cycling, gone;
     GList *li;
 
-    TRACE ("entering clientCycleEventFilter");
+    TRACE ("entering");
 
     passdata = (ClientCycleData *) data;
     c = tabwinGetSelected(passdata->tabwin);
@@ -267,81 +272,70 @@ clientCycleEventFilter (XEvent * xevent, gpointer data)
     down = screen_info->params->keys[KEY_DOWN].keycode;
     modifiers = (screen_info->params->keys[KEY_CYCLE_WINDOWS].modifier |
                  screen_info->params->keys[KEY_CYCLE_REVERSE_WINDOWS].modifier);
-    status = EVENT_FILTER_STOP;
+    status = EVENT_FILTER_CONTINUE;
     removed = NULL;
     cycling = TRUE;
     gone = FALSE;
 
     /* Update the display time */
-    myDisplayUpdateCurrentTime (display_info, xevent);
+    myDisplayUpdateCurrentTime (display_info, event);
 
-    switch (xevent->type)
+    switch (event->meta.type)
     {
-        case DestroyNotify:
-            status = EVENT_FILTER_CONTINUE;
-            if ((removed = myScreenGetClientFromWindow (screen_info, ((XDestroyWindowEvent *) xevent)->window, SEARCH_WINDOW)) == NULL)
-                break; /* No need to go any further */
-            gone |= (c == removed);
-            /* Walk through */
-        case UnmapNotify:
-            status = EVENT_FILTER_CONTINUE;
-            if (!removed && (removed = myScreenGetClientFromWindow (screen_info, ((XUnmapEvent *) xevent)->window, SEARCH_WINDOW)) == NULL)
-                break; /* No need to go any further */
-            gone |= (c == removed);
-            c = tabwinRemoveClient(passdata->tabwin, removed);
-            cycling = clientCycleUpdateWireframe (c, passdata);
-            break;
-        case KeyPress:
-            key = myScreenGetKeyPressed (screen_info, (XKeyEvent *) xevent);
-            /*
-             * We cannot simply check for key == KEY_CANCEL here because of the
-             * modidier being pressed, so we need to look at the keycode directly.
-             */
-            if (xevent->xkey.keycode == cancel)
+        case XFWM_EVENT_KEY:
+            if (event->key.pressed)
             {
-                c2 = tabwinSelectHead (passdata->tabwin);
-                cycling = FALSE;
-            }
-            else if (xevent->xkey.keycode == up)
-            {
-                c2 = tabwinSelectDelta(passdata->tabwin, -1, 0);
-            }
-            else if (xevent->xkey.keycode == down)
-            {
-                c2 = tabwinSelectDelta(passdata->tabwin, 1, 0);
-            }
-            else if (xevent->xkey.keycode == left)
-            {
-                c2 = tabwinSelectDelta(passdata->tabwin, 0, -1);
-            }
-            else if (xevent->xkey.keycode == right)
-            {
-                c2 = tabwinSelectDelta(passdata->tabwin, -0, 1);
-            }
-            else if (key == KEY_CYCLE_REVERSE_WINDOWS)
-            {
-                TRACE ("Cycle: previous");
-                c2 = tabwinSelectPrev(passdata->tabwin);
-            }
-            else if (key == KEY_CYCLE_WINDOWS)
-            {
-                TRACE ("Cycle: next");
-                c2 = tabwinSelectNext(passdata->tabwin);
-            }
-            if (c2)
-            {
-                clientCycleUpdateWireframe (c2, passdata);
-            }
+                key = myScreenGetKeyPressed (screen_info, &event->key);
+                /*
+                 * We cannot simply check for key == KEY_CANCEL here because of the
+                 * modidier being pressed, so we need to look at the keycode directly.
+                 */
+                if (event->key.keycode == cancel)
+                {
+                    c2 = tabwinSelectHead (passdata->tabwin);
+                    cycling = FALSE;
+                }
+                else if (event->key.keycode == up)
+                {
+                    c2 = tabwinSelectDelta(passdata->tabwin, -1, 0);
+                }
+                else if (event->key.keycode == down)
+                {
+                    c2 = tabwinSelectDelta(passdata->tabwin, 1, 0);
+                }
+                else if (event->key.keycode == left)
+                {
+                    c2 = tabwinSelectDelta(passdata->tabwin, 0, -1);
+                }
+                else if (event->key.keycode == right)
+                {
+                    c2 = tabwinSelectDelta(passdata->tabwin, -0, 1);
+                }
+                else if (key == KEY_CYCLE_REVERSE_WINDOWS)
+                {
+                    TRACE ("cycle: previous");
+                    c2 = tabwinSelectPrev(passdata->tabwin);
+                }
+                else if (key == KEY_CYCLE_WINDOWS)
+                {
+                    TRACE ("cycle: next");
+                    c2 = tabwinSelectNext(passdata->tabwin);
+                }
+                if (c2)
+                {
+                    clientCycleUpdateWireframe (c2, passdata);
+                }
 
-            /* If last key press event had not our modifiers pressed, finish cycling */
-            if (!(xevent->xkey.state & modifiers))
-            {
-                cycling = FALSE;
+                /* If last key press event had not our modifiers pressed, finish cycling */
+                if (!(event->key.state & modifiers))
+                {
+                    cycling = FALSE;
+                }
+                status = EVENT_FILTER_STOP;
             }
-            break;
-        case KeyRelease:
+            else
             {
-                int keysym = XLookupKeysym (&xevent->xkey, 0);
+                int keysym = XkbKeycodeToKeysym (event->meta.x->xany.display, event->key.keycode, 0, 0);
 
                 if (IsModifierKey(keysym))
                 {
@@ -351,53 +345,75 @@ clientCycleEventFilter (XEvent * xevent, gpointer data)
                     }
                 }
             }
+            status = EVENT_FILTER_STOP;
             break;
-        case ButtonPress:
-            /* only accept events for the tab windows */
-            for (li = passdata->tabwin->tabwin_list; li != NULL; li = li->next)
+        case XFWM_EVENT_BUTTON:
+            if (event->button.pressed)
             {
-                if (GDK_WINDOW_XID (gtk_widget_get_window (li->data)) == xevent->xbutton.window)
+                /* only accept events for the tab windows */
+                for (li = passdata->tabwin->tabwin_list; li != NULL; li = li->next)
                 {
-                    if  (xevent->xbutton.button == Button1)
+                    if (GDK_WINDOW_XID (gtk_widget_get_window (li->data)) == event->meta.window)
                     {
-                        c2 = tabwinSelectHovered (passdata->tabwin);
-                        cycling = FALSE;
-                        break;
+                        if  (event->button.button == Button1)
+                        {
+                            c2 = tabwinSelectHovered (passdata->tabwin);
+                            cycling = FALSE;
+                            break;
+                        }
+                        else if (event->button.button == Button4)
+                        {
+                            /* Mouse wheel scroll up */
+                            TRACE ("cycle: previous");
+                            c2 = tabwinSelectPrev(passdata->tabwin);
+                        }
+                        else if (event->button.button == Button5)
+                        {
+                            /* Mouse wheel scroll down */
+                            TRACE ("cycle: next");
+                            c2 = tabwinSelectNext(passdata->tabwin);
+                        }
                     }
-                    else if  (xevent->xbutton.button == Button4)
+                    if (c2)
                     {
-                        /* Mouse wheel scroll up */
-                        TRACE ("Cycle: previous");
-                        c2 = tabwinSelectPrev(passdata->tabwin);
+                        clientCycleUpdateWireframe (c2, passdata);
                     }
-                    else if (xevent->xbutton.button == Button5)
-                    {
-                        /* Mouse wheel scroll down */
-                        TRACE ("Cycle: next");
-                        c2 = tabwinSelectNext(passdata->tabwin);
-                    }
-                }
-                if (c2)
-                {
-                    clientCycleUpdateWireframe (c2, passdata);
                 }
             }
+            status = EVENT_FILTER_STOP;
             break;
-        case ButtonRelease:
+        case XFWM_EVENT_MOTION:
             break;
-        case EnterNotify:
-        case LeaveNotify:
+        case XFWM_EVENT_CROSSING:
             /* Track whether the pointer is inside one of the tab-windows */
             for (li = passdata->tabwin->tabwin_list; li != NULL; li = li->next)
             {
-                if (GDK_WINDOW_XID (gtk_widget_get_window (li->data)) == xevent->xcrossing.window)
+                if (GDK_WINDOW_XID (gtk_widget_get_window (li->data)) == event->meta.window)
                 {
-                    passdata->inside = (xevent->xcrossing.type == EnterNotify);
+                    passdata->inside = event->crossing.enter;
                 }
             }
+            status = EVENT_FILTER_STOP;
             break;
-        default:
-            status = EVENT_FILTER_CONTINUE;
+        case XFWM_EVENT_X:
+            switch (event->meta.x->type)
+            {
+                case DestroyNotify:
+                    status = EVENT_FILTER_CONTINUE;
+                    if ((removed = myScreenGetClientFromWindow (screen_info, ((XDestroyWindowEvent *) event->meta.x)->window, SEARCH_WINDOW)) == NULL)
+                        break; /* No need to go any further */
+                    gone |= (c == removed);
+                    FALLTHROUGH;
+                case UnmapNotify:
+                    status = EVENT_FILTER_CONTINUE;
+                    if (!removed && (removed = myScreenGetClientFromWindow (screen_info, ((XUnmapEvent *) event->meta.x)->window, SEARCH_WINDOW)) == NULL)
+                        break; /* No need to go any further */
+                    gone |= (c == removed);
+                    c = tabwinRemoveClient(passdata->tabwin, removed);
+                    cycling = clientCycleUpdateWireframe (c, passdata);
+                    status = EVENT_FILTER_STOP;
+                    break;
+            }
             break;
     }
 
@@ -411,27 +427,23 @@ clientCycleEventFilter (XEvent * xevent, gpointer data)
 }
 
 static eventFilterStatus
-clientCycleFlushEventFilter (XEvent * xevent, gpointer data)
+clientCycleFlushEventFilter (XfwmEvent *event, gpointer data)
 {
     DisplayInfo *display_info = (DisplayInfo *) data;
 
     /* Update the display time */
-    myDisplayUpdateCurrentTime (display_info, xevent);
+    myDisplayUpdateCurrentTime (display_info, event);
 
-    switch (xevent->type)
+    if (event->meta.type == XFWM_EVENT_CROSSING && event->crossing.enter)
     {
-        case EnterNotify:
-            gtk_main_quit ();
-            return EVENT_FILTER_STOP;
-            break;
-        default:
-            break;
+        gtk_main_quit ();
+        return EVENT_FILTER_STOP;
     }
     return EVENT_FILTER_CONTINUE;
 }
 
 void
-clientCycle (Client * c, XKeyEvent * ev)
+clientCycle (Client * c, XfwmEventKey *event)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -440,7 +452,7 @@ clientCycle (Client * c, XKeyEvent * ev)
     int key, modifier;
 
     g_return_if_fail (c != NULL);
-    TRACE ("entering clientCycle");
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
@@ -452,7 +464,7 @@ clientCycle (Client * c, XKeyEvent * ev)
     }
 
     modifier = 0;
-    key = myScreenGetKeyPressed (screen_info, ev);
+    key = myScreenGetKeyPressed (screen_info, event);
     if (key == KEY_CYCLE_REVERSE_WINDOWS)
     {
         selected = g_list_last (client_list);
@@ -481,13 +493,12 @@ clientCycle (Client * c, XKeyEvent * ev)
         return;
     }
 
-    myScreenGrabPointer (screen_info, TRUE, NoEventMask, None, ev->time);
+    myScreenGrabPointer (screen_info, TRUE, EnterWindowMask | LeaveWindowMask, None, event->time);
     /* Grabbing the pointer may fail e.g. if the user is doing a drag'n drop */
-    if (!myScreenGrabKeyboard (screen_info, ev->time))
+    if (!myScreenGrabKeyboard (screen_info, KeyPressMask | KeyReleaseMask, event->time))
     {
         TRACE ("grab failed in clientCycle");
-
-        gdk_beep ();
+        myDisplayBeep (display_info);
         myScreenUngrabKeyboard (screen_info, myDisplayGetCurrentTime (display_info));
         myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
         g_list_free (client_list);
@@ -499,6 +510,10 @@ clientCycle (Client * c, XKeyEvent * ev)
     passdata.inside = FALSE;
 
     TRACE ("entering cycle loop");
+    if (screen_info->params->cycle_raise)
+    {
+      clientRaise ((Client *) selected->data, None);
+    }
     if (screen_info->params->cycle_draw_frame)
     {
         passdata.wireframe = wireframeCreate ((Client *) selected->data);
@@ -566,7 +581,7 @@ clientSwitchApp (void)
     Client *focus, *new;
     guint range;
 
-    TRACE ("entering clientSwitchApp");
+    TRACE ("entering");
 
     focus = clientGetFocus();
     if (!focus)
