@@ -116,6 +116,7 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_NET_WM_ACTION_SHADE",
         "_NET_WM_ACTION_STICK",
         "_NET_WM_ALLOWED_ACTIONS",
+        "_NET_WM_BYPASS_COMPOSITOR",
         "_NET_WM_CONTEXT_HELP",
         "_NET_WM_DESKTOP",
         "_NET_WM_FULLSCREEN_MONITORS",
@@ -177,7 +178,8 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "XFWM4_COMPOSITING_MANAGER",
         "XFWM4_TIMESTAMP_PROP",
         "_XROOTPMAP_ID",
-        "_XSETROOT_ID"
+        "_XSETROOT_ID",
+        "_GTK_READ_RCFILES"
     };
 
     g_assert (ATOM_COUNT == G_N_ELEMENTS (atom_names));
@@ -230,6 +232,8 @@ myDisplayInit (GdkDisplay *gdisplay)
         g_warning ("Some internal atoms were not properly created.");
     }
 
+    display->devices = xfwm_devices_new (gdisplay);
+
     /* Test XShape extension support */
     major = 0;
     minor = 0;
@@ -367,6 +371,9 @@ myDisplayClose (DisplayInfo *display)
     g_slist_free (display->screens);
     display->screens = NULL;
 
+    g_free (display->devices);
+    display->devices = NULL;
+
     return display;
 }
 
@@ -594,7 +601,7 @@ myDisplayGetScreenFromRoot (DisplayInfo *display, Window root)
             return screen;
         }
     }
-    TRACE ("myDisplayGetScreenFromRoot: no screen found");
+    TRACE ("no screen found");
 
     return NULL;
 }
@@ -602,6 +609,7 @@ myDisplayGetScreenFromRoot (DisplayInfo *display, Window root)
 ScreenInfo *
 myDisplayGetScreenFromOutput (DisplayInfo *display, Window output)
 {
+#ifdef HAVE_COMPOSITOR
     GSList *list;
 
     g_return_val_if_fail (display != NULL, NULL);
@@ -614,7 +622,8 @@ myDisplayGetScreenFromOutput (DisplayInfo *display, Window output)
             return screen;
         }
     }
-    TRACE ("myDisplayGetScreenFromOutput: no screen found for output window %p", output);
+    TRACE ("no screen found for output window 0x%lx", output);
+#endif
 
     return NULL;
 }
@@ -635,7 +644,7 @@ myDisplayGetScreenFromNum (DisplayInfo *display, int num)
             return screen;
         }
     }
-    TRACE ("myDisplayGetScreenFromNum: no screen found");
+    TRACE ("no screen found");
 
     return NULL;
 }
@@ -650,7 +659,7 @@ myDisplayGetRootFromWindow(DisplayInfo *display, Window w)
 
     if (!XGetWindowAttributes(display->dpy, w, &attributes))
     {
-        TRACE ("myDisplayGetRootFromWindow: no root found for 0x%lx", w);
+        TRACE ("no root found for 0x%lx", w);
         return None;
     }
     return attributes.root;
@@ -682,7 +691,7 @@ myDisplayGetScreenFromWindow (DisplayInfo *display, Window w)
             return screen;
         }
     }
-    TRACE ("myDisplayGetScreenFromWindow: no screen found for 0x%lx", w);
+    TRACE ("no screen found for 0x%lx", w);
 
     return NULL;
 }
@@ -704,7 +713,7 @@ myDisplayGetScreenFromSystray (DisplayInfo *display, Window w)
             return screen;
         }
     }
-    TRACE ("myDisplayGetScreenFromSystray: no screen found");
+    TRACE ("no screen found");
 
     return NULL;
 }
@@ -750,50 +759,70 @@ myDisplayGetDefaultScreen (DisplayInfo *display)
 }
 
 guint32
-myDisplayUpdateCurrentTime (DisplayInfo *display, XEvent *ev)
+myDisplayUpdateCurrentTime (DisplayInfo *display, XfwmEvent *event)
 {
     guint32 timestamp;
+    XEvent *xevent;
 
     g_return_val_if_fail (display != NULL, (guint32) CurrentTime);
 
     timestamp = (guint32) CurrentTime;
-    switch (ev->type)
+    switch (event->meta.type)
     {
-        case KeyPress:
-        case KeyRelease:
-            timestamp = (guint32) ev->xkey.time;
+        case XFWM_EVENT_KEY:
+            timestamp = (guint32) event->key.time;
             break;
-        case ButtonPress:
-        case ButtonRelease:
-            timestamp = (guint32) ev->xbutton.time;
+        case XFWM_EVENT_BUTTON:
+            timestamp = (guint32) event->button.time;
             break;
-        case MotionNotify:
-            timestamp = (guint32) ev->xmotion.time;
+        case XFWM_EVENT_MOTION:
+            timestamp = (guint32) event->motion.time;
             break;
-        case EnterNotify:
-        case LeaveNotify:
-            timestamp = (guint32) ev->xcrossing.time;
+        case XFWM_EVENT_CROSSING:
+            timestamp = (guint32) event->crossing.time;
             break;
-        case PropertyNotify:
-            timestamp = (guint32) ev->xproperty.time;
-            break;
-        case SelectionClear:
-            timestamp = (guint32) ev->xselectionclear.time;
-            break;
-        case SelectionRequest:
-            timestamp = (guint32) ev->xselectionrequest.time;
-            break;
-        case SelectionNotify:
-            timestamp = (guint32) ev->xselection.time;
-            break;
-        default:
-#ifdef HAVE_XSYNC
-            if ((display->have_xsync) && (ev->type == display->xsync_event_base + XSyncAlarmNotify))
+        case XFWM_EVENT_X:
+            xevent = event->meta.x;
+
+            switch (xevent->type)
             {
-                timestamp = ((XSyncAlarmNotifyEvent*) ev)->time;
-            }
+                case KeyPress:
+                case KeyRelease:
+                    timestamp = (guint32) xevent->xkey.time;
+                    break;
+                case ButtonPress:
+                case ButtonRelease:
+                    timestamp = (guint32) xevent->xbutton.time;
+                    break;
+                case MotionNotify:
+                    timestamp = (guint32) xevent->xmotion.time;
+                    break;
+                case EnterNotify:
+                case LeaveNotify:
+                    timestamp = (guint32) xevent->xcrossing.time;
+                    break;
+                case PropertyNotify:
+                    timestamp = (guint32) xevent->xproperty.time;
+                    break;
+                case SelectionClear:
+                    timestamp = (guint32) xevent->xselectionclear.time;
+                    break;
+                case SelectionRequest:
+                    timestamp = (guint32) xevent->xselectionrequest.time;
+                    break;
+                case SelectionNotify:
+                    timestamp = (guint32) xevent->xselection.time;
+                    break;
+                default:
+#ifdef HAVE_XSYNC
+                    if ((display->have_xsync) &&
+                        (xevent->type == display->xsync_event_base + XSyncAlarmNotify))
+                    {
+                        timestamp = ((XSyncAlarmNotifyEvent*) xevent)->time;
+                    }
 #endif /* HAVE_XSYNC */
-            break;
+                    break;
+            }
     }
 
     if ((timestamp != (guint32) CurrentTime))
@@ -809,7 +838,7 @@ myDisplayGetCurrentTime (DisplayInfo *display)
 {
     g_return_val_if_fail (display != NULL, (guint32) CurrentTime);
 
-    TRACE ("myDisplayGetCurrentTime gives timestamp=%u", (guint32) display->current_time);
+    TRACE ("timestamp=%u", (guint32) display->current_time);
     return display->current_time;
 }
 
@@ -824,7 +853,7 @@ myDisplayGetTime (DisplayInfo * display, guint32 timestamp)
         display_timestamp = getXServerTime (display);
     }
 
-    TRACE ("myDisplayGetTime gives timestamp=%u", (guint32) display_timestamp);
+    TRACE ("timestamp=%u", (guint32) display_timestamp);
     return display_timestamp;
 }
 
@@ -833,7 +862,7 @@ myDisplayGetLastUserTime (DisplayInfo *display)
 {
     g_return_val_if_fail (display != NULL, (guint32) CurrentTime);
 
-    TRACE ("myDisplayGetLastUserTime gives timestamp=%u", (guint32) display->last_user_time);
+    TRACE ("timestamp=%u", (guint32) display->last_user_time);
     return display->last_user_time;
 }
 
@@ -885,7 +914,7 @@ myDisplayTestXrender (DisplayInfo *display, gdouble min_time)
     int iterations;
 
     g_return_val_if_fail (display != NULL, FALSE);
-    TRACE ("entering myDisplayTesxrender");
+    TRACE ("entering");
 
     c.alpha = 0x7FFF;
     c.red   = 0xFFFF;
@@ -992,3 +1021,56 @@ myDisplayTestXrender (DisplayInfo *display, gdouble min_time)
     return FALSE;
 #endif /* HAVE_RENDER */
 }
+
+void
+myDisplayErrorTrapPush (DisplayInfo *display_info)
+{
+#if GTK_CHECK_VERSION(3, 22, 0)
+    gdk_x11_display_error_trap_push (display_info->gdisplay);
+#else
+    gdk_error_trap_push ();
+#endif
+}
+
+gint
+myDisplayErrorTrapPop (DisplayInfo *display_info)
+{
+#if GTK_CHECK_VERSION(3, 22, 0)
+    gdk_display_flush (display_info->gdisplay);
+    return gdk_x11_display_error_trap_pop (display_info->gdisplay);
+#else
+    gdk_flush ();
+    return gdk_error_trap_pop ();
+#endif
+}
+
+void
+myDisplayErrorTrapPopIgnored (DisplayInfo *display_info)
+{
+#if GTK_CHECK_VERSION(3, 22, 0)
+    gdk_x11_display_error_trap_pop_ignored (display_info->gdisplay);
+#else
+    gdk_error_trap_pop_ignored ();
+#endif
+}
+
+void
+myDisplayBeep (DisplayInfo *display_info)
+{
+#if GTK_CHECK_VERSION(3, 22, 0)
+    gdk_display_beep (display_info->gdisplay);;
+#else
+    gdk_beep ();
+#endif
+}
+
+GdkKeymap *
+myDisplayGetKeymap (DisplayInfo *display_info)
+{
+#if GTK_CHECK_VERSION(3, 22, 0)
+    return gdk_keymap_get_for_display (display_info->gdisplay);
+#else
+    return gdk_keymap_get_default ();
+#endif
+}
+
