@@ -208,6 +208,7 @@ enum
     NET_WM_ACTION_SHADE,
     NET_WM_ACTION_STICK,
     NET_WM_ALLOWED_ACTIONS,
+    NET_WM_BYPASS_COMPOSITOR,
     NET_WM_CONTEXT_HELP,
     NET_WM_DESKTOP,
     NET_WM_FULLSCREEN_MONITORS,
@@ -270,12 +271,12 @@ enum
     XFWM4_TIMESTAMP_PROP,
     XROOTPMAP,
     XSETROOT,
+    GTK_READ_RCFILES,
     ATOM_COUNT
 };
 
 typedef struct _Client            Client;
 typedef struct _DisplayInfo       DisplayInfo;
-typedef struct _XfwmColor         XfwmColor;
 typedef struct _xfwmPixmap        xfwmPixmap;
 typedef struct _XfwmParams        XfwmParams;
 typedef struct _ScreenInfo        ScreenInfo;
@@ -299,6 +300,7 @@ struct _DisplayInfo
     Atom atoms[ATOM_COUNT];
 
     eventFilterSetup *xfilter;
+    XfwmDevices *devices;
     GSList *screens;
     GSList *clients;
 
@@ -318,6 +320,8 @@ struct _DisplayInfo
     guint32 last_user_time;
 
     gboolean enable_compositor;
+    gint vblank_method;
+
 #ifdef HAVE_RENDER
     gint render_error_base;
     gint render_event_base;
@@ -407,7 +411,7 @@ Client                  *myDisplayGetClientFromXSyncAlarm       (DisplayInfo *,
 #endif /* HAVE_XSYNC */
 ScreenInfo              *myDisplayGetDefaultScreen              (DisplayInfo *);
 guint32                  myDisplayUpdateCurrentTime             (DisplayInfo *,
-                                                                 XEvent *);
+                                                                 XfwmEvent *);
 guint32                  myDisplayGetCurrentTime                (DisplayInfo *);
 guint32                  myDisplayGetTime                       (DisplayInfo *,
                                                                  guint32);
@@ -418,5 +422,10 @@ void                     myDisplayUpdateLastUserTime            (DisplayInfo *,
                                                                  guint32);
 gboolean                 myDisplayTestXrender                   (DisplayInfo *,
                                                                  gdouble);
+void                     myDisplayErrorTrapPush                 (DisplayInfo *);
+gint                     myDisplayErrorTrapPop                  (DisplayInfo *);
+void                     myDisplayErrorTrapPopIgnored           (DisplayInfo *);
+void                     myDisplayBeep                          (DisplayInfo *);
+GdkKeymap               *myDisplayGetKeymap                     (DisplayInfo *);
 
 #endif /* INC_DISPLAY_H */
