@@ -31,9 +31,12 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <cairo/cairo-xlib.h>
+#ifdef HAVE_COMPOSITOR
+#include <cairo/cairo-xlib-xrender.h>
+#endif
 #include <libxfce4util/libxfce4util.h>
 
-#include "inline-default-icon.h"
 #include "icons.h"
 #include "display.h"
 #include "screen.h"
@@ -70,34 +73,33 @@ downsize_ratio (guint *width, guint *height, guint dest_w, guint dest_h)
  * create a GdkPixbuf from inline data and scale it to a given size
  */
 static GdkPixbuf *
-inline_icon_at_size (const guint8 *data, guint width, guint height)
+default_icon_at_size (GdkScreen *screen, guint width, guint height)
 {
+    GtkIconTheme *icon_theme;
     GdkPixbuf *base;
+    GdkPixbuf *scaled;
 
-    base = gdk_pixbuf_new_from_inline (-1, data, FALSE, NULL);
+    icon_theme = gtk_icon_theme_get_for_screen (screen);
 
-    g_return_val_if_fail (base, NULL);
+    g_return_val_if_fail (icon_theme != NULL, NULL);
 
-    if ((width <= 0 || height <= 0) ||
-        ((guint) gdk_pixbuf_get_width (base) == width &&
-         (guint) gdk_pixbuf_get_height (base) == height))
+    if (width <= 0 || height <= 0)
     {
-        return base;
+        width = 160;
+        height = 160;
     }
-    else
+
+    base = gtk_icon_theme_load_icon (icon_theme, "xfwm4-default",
+                                     MAX (width, height), 0, NULL);
+
+    if (base != NULL && width != height)
     {
-        GdkPixbuf *scaled;
-        guint w, h;
-
-        w = gdk_pixbuf_get_width (base);
-        h = gdk_pixbuf_get_height (base);
-        downsize_ratio (&w, &h, width, height);
-        scaled = gdk_pixbuf_scale_simple (base, w, h, GDK_INTERP_BILINEAR);
-
+        scaled = gdk_pixbuf_scale_simple (base, width, height, GDK_INTERP_BILINEAR);
         g_object_unref (G_OBJECT (base));
-
         return scaled;
     }
+
+    return base;
 }
 
 
@@ -293,14 +295,28 @@ read_rgb_icon (DisplayInfo *display_info, Window window, guint ideal_width, guin
 }
 
 static void
-get_pixmap_geometry (Display *dpy, Pixmap pixmap, guint *w, guint *h)
+get_pixmap_geometry (Display *dpy, Pixmap pixmap, guint *out_width, guint *out_height, guint *out_depth)
 {
     Window root;
     guint border_width;
+    gint x, y;
+    guint width, height;
     guint depth;
-    int x, y;
 
-    XGetGeometry (dpy, pixmap, &root, &x, &y, w, h, &border_width, &depth);
+    XGetGeometry (dpy, pixmap, &root, &x, &y, &width, &height, &border_width, &depth);
+
+    if (out_width != NULL)
+    {
+        *out_width = width;
+    }
+    if (out_height != NULL)
+    {
+        *out_height = height;
+    }
+    if (out_depth != NULL)
+    {
+        *out_depth = depth;
+    }
 }
 
 static GdkPixbuf *
@@ -311,6 +327,7 @@ apply_mask (GdkPixbuf * pixbuf, GdkPixbuf * mask)
     guchar *dest;
     guint w, h, i, j;
     guint src_stride, dest_stride;
+    guint src_bpx, dest_bpx;
 
     w = MIN (gdk_pixbuf_get_width (mask), gdk_pixbuf_get_width (pixbuf));
     h = MIN (gdk_pixbuf_get_height (mask), gdk_pixbuf_get_height (pixbuf));
@@ -323,26 +340,24 @@ apply_mask (GdkPixbuf * pixbuf, GdkPixbuf * mask)
     dest_stride = gdk_pixbuf_get_rowstride (with_alpha);
     src_stride = gdk_pixbuf_get_rowstride (mask);
 
+    dest_bpx = dest_stride / gdk_pixbuf_get_width (with_alpha);
+    src_bpx = src_stride / gdk_pixbuf_get_width (mask);
+
+    if (G_UNLIKELY (dest_bpx != 4))
+    {
+        g_object_unref (with_alpha);
+        g_return_val_if_reached (NULL);
+    }
+
     i = 0;
     while (i < h)
     {
         j = 0;
         while (j < w)
         {
-            guchar *s = src + i * src_stride + j * 3;
-            guchar *d = dest + i * dest_stride + j * 4;
-
-            /* s[0] == s[1] == s[2], they are 255 if the bit was set, 0
-             * otherwise
-             */
-            if (s[0] == 0)
-            {
-                d[3] = 0;       /* transparent */
-            }
-            else
-            {
-                d[3] = 255;     /* opaque */
-            }
+            guchar *s = src + i * src_stride + j * src_bpx;
+            guchar *d = dest + i * dest_stride + j * dest_bpx;
+            d[dest_bpx - 1] = s[src_bpx - 1];
             ++j;
         }
         ++i;
@@ -351,86 +366,53 @@ apply_mask (GdkPixbuf * pixbuf, GdkPixbuf * mask)
     return with_alpha;
 }
 
-static GdkColormap *
-get_cmap (GdkPixmap * pixmap, GdkScreen *gscreen)
-{
-    GdkColormap *cmap;
-
-    g_return_val_if_fail (pixmap != NULL, NULL);
-
-    cmap = gdk_drawable_get_colormap (pixmap);
-    if (cmap)
-    {
-        g_message ("Drawable colormap");
-        g_object_ref (G_OBJECT (cmap));
-    }
-    else
-    {
-        if (gdk_drawable_get_depth (pixmap) == 1)
-        {
-            /* try null cmap */
-            cmap = NULL;
-        }
-        else if ((gdk_drawable_get_depth (pixmap) == 32) && (gscreen != NULL))
-        {
-            /* Try ARGB cmap */
-            cmap = gdk_screen_get_rgba_colormap(gscreen);
-            g_object_ref (G_OBJECT (cmap));
-        }
-        else
-        {
-            /* Try system cmap */
-            cmap = gdk_colormap_get_system ();
-            g_object_ref (G_OBJECT (cmap));
-        }
-    }
-
-    /* Be sure we aren't going to blow up due to visual mismatch */
-    if (cmap && (gdk_colormap_get_visual (cmap)->depth != gdk_drawable_get_depth (pixmap)))
-    {
-        g_object_unref (G_OBJECT (cmap));
-        cmap = NULL;
-    }
-
-    return cmap;
-}
-
 static GdkPixbuf *
-get_pixbuf_from_pixmap (GdkScreen *gscreen, Pixmap xpixmap, guint src_x, guint src_y,
-                        gint dest_x, gint dest_y, guint width, guint height)
+get_pixbuf_from_pixmap (ScreenInfo *screen_info, Pixmap xpixmap, guint width, guint height, guint depth)
 {
-    GdkDrawable *drawable;
+    cairo_surface_t *surface;
     GdkPixbuf *retval;
-    GdkColormap *cmap;
+#ifdef HAVE_COMPOSITOR
+    XRenderPictFormat *render_format;
+#endif
 
     retval = NULL;
 
-    drawable = gdk_xid_table_lookup (xpixmap);
-
-    if (drawable)
+    if (depth == 1)
     {
-        g_object_ref (G_OBJECT (drawable));
+        surface = cairo_xlib_surface_create_for_bitmap (screen_info->display_info->dpy,
+                                                        xpixmap,
+                                                        screen_info->xscreen,
+                                                        width, height);
     }
+#ifdef HAVE_COMPOSITOR
+    else if (depth == 32)
+    {
+        render_format = XRenderFindStandardFormat (screen_info->display_info->dpy,
+                                                   PictStandardARGB32);
+        surface = cairo_xlib_surface_create_with_xrender_format (screen_info->display_info->dpy,
+                                                                 xpixmap,
+                                                                 screen_info->xscreen,
+                                                                 render_format,
+                                                                 width, height);
+    }
+#endif
     else
     {
-        drawable = gdk_pixmap_foreign_new (xpixmap);
+        surface = cairo_xlib_surface_create (screen_info->display_info->dpy,
+                                             xpixmap,
+                                             screen_info->visual,
+                                             width, height);
     }
 
-    if (G_UNLIKELY(!drawable))
+    if (G_UNLIKELY(!surface))
     {
         /* Pixmap is gone ?? */
         return NULL;
     }
 
-    cmap = get_cmap (drawable, gscreen);
-    retval = gdk_pixbuf_get_from_drawable (NULL, drawable, cmap, src_x, src_y,
-                                           dest_x, dest_y, width, height);
+    retval = gdk_pixbuf_get_from_surface (surface, 0, 0, width, height);
 
-    if (G_LIKELY(cmap))
-    {
-        g_object_unref (G_OBJECT (cmap));
-    }
-    g_object_unref (G_OBJECT (drawable));
+    cairo_surface_destroy (surface);
 
     return retval;
 }
@@ -441,33 +423,37 @@ try_pixmap_and_mask (ScreenInfo *screen_info, Pixmap src_pixmap, Pixmap src_mask
     GdkPixbuf *unscaled;
     GdkPixbuf *icon;
     GdkPixbuf *mask;
-    guint w, h;
+    guint w, h, depth;
 
     if (src_pixmap == None)
     {
         return NULL;
     }
 
-    gdk_error_trap_push ();
-    get_pixmap_geometry (myScreenGetXDisplay(screen_info), src_pixmap, &w, &h);
-    unscaled = get_pixbuf_from_pixmap (screen_info->gscr, src_pixmap, 0, 0, 0, 0, w, h);
+    myDisplayErrorTrapPush (screen_info->display_info);
+    get_pixmap_geometry (myScreenGetXDisplay(screen_info), src_pixmap, &w, &h, &depth);
+    unscaled = get_pixbuf_from_pixmap (screen_info, src_pixmap, w, h, depth);
     icon = NULL;
     mask = NULL;
 
     if (unscaled && src_mask)
     {
-        get_pixmap_geometry (myScreenGetXDisplay(screen_info), src_mask, &w, &h);
-        mask = get_pixbuf_from_pixmap (screen_info->gscr, src_mask, 0, 0, 0, 0, w, h);
+        get_pixmap_geometry (myScreenGetXDisplay(screen_info), src_mask, &w, &h, &depth);
+        mask = get_pixbuf_from_pixmap (screen_info, src_mask, w, h, depth);
     }
-    gdk_error_trap_pop ();
+    myDisplayErrorTrapPopIgnored (screen_info->display_info);
 
     if (mask)
     {
         GdkPixbuf *masked;
 
         masked = apply_mask (unscaled, mask);
-        g_object_unref (G_OBJECT (unscaled));
-        unscaled = masked;
+
+        if (masked != NULL)
+        {
+            g_object_unref (G_OBJECT (unscaled));
+            unscaled = masked;
+        }
 
         g_object_unref (G_OBJECT (mask));
         mask = NULL;
@@ -535,9 +521,9 @@ getAppIcon (ScreenInfo *screen_info, Window window, guint width, guint height)
         return scaled_from_pixdata (pixdata, w, h, width, height);
     }
 
-    gdk_error_trap_push ();
+    myDisplayErrorTrapPush (screen_info->display_info);
     hints = XGetWMHints (myScreenGetXDisplay(screen_info), window);
-    gdk_error_trap_pop ();
+    myDisplayErrorTrapPopIgnored (screen_info->display_info);
 
     if (hints)
     {
@@ -573,7 +559,7 @@ getAppIcon (ScreenInfo *screen_info, Window window, guint width, guint height)
         }
     }
 
-    return inline_icon_at_size (default_icon_data, width, height);
+    return default_icon_at_size (screen_info->gscr, width, height);
 }
 
 GdkPixbuf *
@@ -606,7 +592,7 @@ getClientIcon (Client *c, guint width, guint height)
     }
     else
     {
-        app_content = inline_icon_at_size (default_icon_data, width, height);
+        app_content = default_icon_at_size (screen_info->gscr, width, height);
     }
 
     app_icon_width = (guint) gdk_pixbuf_get_width (app_content);
