@@ -89,12 +89,20 @@ enum {
 };
 
 static gint compositor = COMPOSITOR_MODE_MANUAL;
+static gint vblank_method = VBLANK_AUTO;
+#define XFWM4_ERROR      (xfwm4_error_quark ())
 
 #ifndef DEBUG
 /* For what, IEEE Std 1003.1-2001, Section 12.2, Utility Syntax Guidelines.*/
 static char revision[]="@(#)$ " PACKAGE " version " VERSION " revision " REVISION " $";
 #endif
 
+static GQuark
+xfwm4_error_quark (void)
+{
+  return g_quark_from_static_string ("xfwm4-error-quark");
+}
+
 #ifdef DEBUG
 static gboolean
 setupLog (void)
@@ -187,7 +195,7 @@ cleanUp (void)
     GSList *screens;
     DisplayInfo *display_info;
 
-    TRACE ("entering cleanUp");
+    TRACE ("entering");
 
     setupHandler (FALSE);
 
@@ -419,17 +427,6 @@ get_default_compositor (DisplayInfo *display_info)
         return 0;
     }
 
-#if 0
-    /*
-     * Test if the XRender implementation is fast enough for the
-     * compositor.
-     */
-    if (!myDisplayTestXrender (display_info, 0.025))
-    {
-        g_warning ("The XRender implementation currently in use on this system is too slow for the compositor");
-        return 0;
-    }
-#endif
     return 2;
 }
 #endif /* HAVE_COMPOSITOR */
@@ -459,7 +456,44 @@ compositor_callback (const gchar  *name,
     }
     else
     {
-        g_set_error (error, 0, 0, "Unrecognized compositor option \"%s\"", value);
+        g_set_error (error, XFWM4_ERROR, 0, "Unrecognized compositor option \"%s\"", value);
+        succeed = FALSE;
+    }
+
+    return succeed;
+}
+
+static gboolean
+vblank_callback (const gchar  *name,
+                     const gchar  *value,
+                     gpointer      user_data,
+                     GError      **error)
+{
+    gboolean succeed = TRUE;
+
+    g_return_val_if_fail (value != NULL, FALSE);
+
+#ifdef HAVE_PRESENT_EXTENSION
+    if (strcmp (value, "xpresent") == 0)
+    {
+        vblank_method = VBLANK_XPRESENT;
+    }
+    else
+#endif /* HAVE_PRESENT_EXTENSION */
+#ifdef HAVE_EPOXY
+    if (strcmp (value, "glx") == 0)
+    {
+        vblank_method = VBLANK_GLX;
+    }
+    else
+#endif /* HAVE_EPOXY */
+    if (strcmp (value, "off") == 0)
+    {
+        vblank_method = VBLANK_OFF;
+    }
+    else
+    {
+        g_set_error (error, XFWM4_ERROR, 0, "Unrecognized compositor option \"%s\"", value);
         succeed = FALSE;
     }
 
@@ -481,9 +515,7 @@ static int
 initialize (gint compositor_mode, gboolean replace_wm)
 {
     DisplayInfo *display_info;
-    gint i, nscreens;
-
-    TRACE ("entering initialize");
+    gint i, nscreens, default_screen;
 
     DBG ("xfwm4 starting, using GTK+-%d.%d.%d", gtk_major_version,
          gtk_minor_version, gtk_micro_version);
@@ -505,6 +537,7 @@ initialize (gint compositor_mode, gboolean replace_wm)
     {
         display_info->enable_compositor = FALSE;
     }
+    display_info->vblank_method = vblank_method;
     compositorSetCompositeMode (display_info, (compositor_mode == COMPOSITOR_MODE_MANUAL));
 #else /* HAVE_COMPOSITOR */
     display_info->enable_compositor = FALSE;
@@ -514,13 +547,41 @@ initialize (gint compositor_mode, gboolean replace_wm)
 
     setupHandler (TRUE);
 
-    nscreens = gdk_display_get_n_screens (display_info->gdisplay);
+    nscreens = ScreenCount (display_info->dpy);
+    default_screen = DefaultScreen (display_info->dpy);
     for(i = 0; i < nscreens; i++)
     {
         ScreenInfo *screen_info;
         GdkScreen *gscr;
+        Window temp_xwindow;
+        GdkWindow *screen_window;
 
-        gscr = gdk_display_get_screen (display_info->gdisplay, i);
+        if (i == default_screen)
+        {
+            gscr = gdk_display_get_default_screen (display_info->gdisplay);
+        }
+        else
+        {
+            /* create temp 1x1 child window on this screen */
+            temp_xwindow = XCreateSimpleWindow (display_info->dpy, RootWindow (display_info->dpy, i),
+                                                0, 0, 1, 1, 0, 0, 0);
+            /* allocate new GdkWindow with GdkScreen for this window */
+            screen_window = gdk_x11_window_foreign_new_for_display (display_info->gdisplay,
+                                                                    temp_xwindow);
+            XDestroyWindow (display_info->dpy, temp_xwindow);
+
+            if (screen_window == NULL)
+            {
+                g_warning ("Cannot create GdkScreen for screen %i", i);
+                continue;
+            }
+
+            gscr = gdk_window_get_screen (screen_window);
+
+            /* foreign windows have 2 references */
+            g_object_unref (screen_window);
+            g_object_unref (screen_window);
+        }
         screen_info = myScreenInit (display_info, gscr, MAIN_EVENT_MASK, replace_wm);
 
         if (!screen_info)
@@ -587,7 +648,7 @@ initialize (gint compositor_mode, gboolean replace_wm)
     {
         return -1;
     }
-    display_info->xfilter = eventFilterInit ((gpointer) display_info);
+    display_info->xfilter = eventFilterInit (display_info->devices, (gpointer) display_info);
     eventFilterPush (display_info->xfilter, xfwm4_event_filter, (gpointer) display_info);
     initPerDisplayCallbacks (display_info);
 
@@ -630,8 +691,7 @@ main (int argc, char **argv)
         { "daemon", '\0', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, daemon_callback, N_("Fork to the background (not supported)"), NULL },
 #ifdef HAVE_COMPOSITOR
         { "compositor", '\0', 0, G_OPTION_ARG_CALLBACK, compositor_callback, N_("Set the compositor mode"), "on|off|auto" },
-#else
-        { "compositor", '\0', 0, G_OPTION_ARG_STRING, &compositor_foo, N_("Set the compositor mode (not supported)"), "on|off|auto" },
+        { "vblank", '\0', 0, G_OPTION_ARG_CALLBACK, vblank_callback, N_("Set the vblank mode"), "off|present|glx" },
 #endif
         { "replace", '\0', 0, G_OPTION_ARG_NONE, &replace_wm, N_("Replace the existing window manager"), NULL },
         { "version", 'V', 0, G_OPTION_ARG_NONE, &version, N_("Print version information and exit"), NULL },
@@ -645,6 +705,17 @@ main (int argc, char **argv)
 
     xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
+    /* xfwm4 is an X11 window manager, no point in trying to connect to
+     * any other display server (like when running nested within a
+     * Wayland compositor).
+     */
+    gdk_set_allowed_backends ("x11");
+
+#ifndef HAVE_XI2
+    /* Disable XI2 in GDK */
+    gdk_disable_multidevice ();
+#endif
+
     context = g_option_context_new (_("[ARGUMENTS...]"));
     g_option_context_add_main_entries (context, option_entries, GETTEXT_PACKAGE);
     g_option_context_add_group (context, gtk_get_option_group (FALSE));
@@ -691,7 +762,7 @@ main (int argc, char **argv)
             gtk_main ();
             break;
         default:
-            g_warning ("Unknown error occured");
+            g_warning ("Unknown error occurred");
             exit (1);
             break;
     }
