@@ -34,10 +34,18 @@
 #include <string.h>
 #include <libxfce4util/libxfce4util.h>
 
+#include <common/xfwm-common.h>
+
 #include "event_filter.h"
 #include "menu.h"
 #include "misc.h"
 
+enum
+{
+    GRAB_DEVICE_POINTER,
+    GRAB_DEVICE_KEYBOARD
+};
+
 static GtkWidget *menu_open = NULL;
 static MenuItem menuitems[] = {
     {MENU_OP_MAXIMIZE,     "xfce-wm-maximize",   N_("Ma_ximize")},
@@ -72,78 +80,78 @@ static MenuItem menuitems[] = {
 };
 
 static eventFilterStatus
-menu_filter (XEvent * xevent, gpointer data)
+menu_filter (XfwmEvent *event, gpointer data)
 {
-    switch (xevent->type)
+    switch (event->meta.type)
     {
-        case KeyPress:
-        case KeyRelease:
-        case ButtonPress:
-        case ButtonRelease:
-        case MotionNotify:
-        case EnterNotify:
-        case LeaveNotify:
+        case XFWM_EVENT_KEY:
+        case XFWM_EVENT_BUTTON:
+        case XFWM_EVENT_MOTION:
+        case XFWM_EVENT_CROSSING:
             return EVENT_FILTER_STOP;
-            break;
-        default:
+        case XFWM_EVENT_X:
             break;
     }
     return EVENT_FILTER_CONTINUE;
 }
 
 
+#if !GTK_CHECK_VERSION(3, 22, 0)
 static void
 popup_position_func (GtkMenu * menu, gint * x, gint * y, gboolean * push_in,
     gpointer user_data)
 {
     GtkRequisition req;
     GdkPoint *pos;
-    gint monitor;
     GdkScreen *screen;
+    gint monitor_num;
+    gint width;
+    gint height;
 
     pos = user_data;
 
-    gtk_widget_size_request (GTK_WIDGET (menu), &req);
+    gtk_widget_get_preferred_size (GTK_WIDGET (menu), NULL, &req);
+
+    xfwm_get_screen_dimensions (&width, &height);
 
     if (pos->x >= 0)
     {
         *x = pos->x;
-        *x = CLAMP (*x, 0, MAX (0, gdk_screen_width () - req.width));
+        *x = CLAMP (*x, 0, MAX (0, width - req.width));
     }
     else
     {
-        *x = (gdk_screen_width () - req.width) / 2;
+        *x = (width - req.width) / 2;
     }
     if (pos->x >= 0)
     {
         *y = pos->y;
-        *y = CLAMP (*y, 0, MAX (0, gdk_screen_height () - req.height));
+        *y = CLAMP (*y, 0, MAX (0, height - req.height));
     }
     else
     {
-        *y = (gdk_screen_height () - req.height) / 2;
+        *y = (height - req.height) / 2;
     }
 
-    screen = gtk_widget_get_screen (GTK_WIDGET(menu));
-    monitor = gdk_screen_get_monitor_at_point (screen, *x, *y);
-    gtk_menu_set_monitor (GTK_MENU (menu), monitor);
+    screen = gtk_widget_get_screen (GTK_WIDGET (menu));
+    monitor_num = gdk_screen_get_monitor_at_point (screen, *x, *y);
+    gtk_menu_set_monitor (GTK_MENU (menu), monitor_num);
 
     g_free (user_data);
 }
+#endif
 
 static gboolean
 activate_cb (GtkWidget * menuitem, gpointer data)
 {
     MenuData *menudata;
 
-    TRACE ("entering activate_cb");
     g_return_val_if_fail (GTK_IS_WIDGET (menuitem), FALSE);
+    TRACE ("entering");
 
     menu_open = NULL;
-
     menudata = data;
 
-    TRACE ("deactivating menu_filter");
     eventFilterPop (menudata->menu->filter_setup);
     (*menudata->menu->func) (menudata->menu,
                              menudata->op,
@@ -158,10 +166,11 @@ menu_closed (GtkMenu * widget, gpointer data)
 {
     Menu *menu;
 
-    TRACE ("entering menu_closed");
+    TRACE ("entering");
+
     menu = data;
     menu_open = NULL;
-    TRACE ("deactivating menu_filter");
+
     eventFilterPop (menu->filter_setup);
     (*menu->func) (menu, 0, menu->xid, menu->data, NULL);
     return (FALSE);
@@ -237,7 +246,8 @@ menu_default (GdkScreen *gscr, Window xid, MenuOp ops, MenuOp insensitive, MenuF
     const gchar *label;
     int i;
 
-    TRACE ("entering menu_new");
+    TRACE ("entering");
+
     menu = g_new (Menu, 1);
     menu->func = func;
     menu->filter_setup = filter_setup;
@@ -269,17 +279,21 @@ menu_default (GdkScreen *gscr, Window xid, MenuOp ops, MenuOp insensitive, MenuF
                     }
                     ws_menu = menu_workspace (menu, insensitive, ws, nws, wsn, wsn_items);
                     gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), ws_menu);
-                    g_signal_connect (GTK_OBJECT (ws_menu), "selection-done", GTK_SIGNAL_FUNC (menu_closed), menu);
+                    g_signal_connect (G_OBJECT (ws_menu), "selection-done", G_CALLBACK (menu_closed), menu);
                     break;
                 default:
                     if (menuitems[i].image_name)
                     {
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
                         menuitem = gtk_image_menu_item_new_with_mnemonic (label);
+G_GNUC_END_IGNORE_DEPRECATIONS
                         image =
                             gtk_image_new_from_icon_name (menuitems[i].image_name,
                             GTK_ICON_SIZE_MENU);
                         gtk_widget_show (image);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
                         gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
                     }
                     else
                     {
@@ -301,7 +315,7 @@ menu_default (GdkScreen *gscr, Window xid, MenuOp ops, MenuOp insensitive, MenuF
         }
         ++i;
     }
-    g_signal_connect (GTK_OBJECT (menu->menu), "selection-done", GTK_SIGNAL_FUNC (menu_closed), menu);
+    g_signal_connect (G_OBJECT (menu->menu), "selection-done", G_CALLBACK (menu_closed), menu);
 
     return (menu);
 }
@@ -309,7 +323,8 @@ menu_default (GdkScreen *gscr, Window xid, MenuOp ops, MenuOp insensitive, MenuF
 static void
 closure_notify (gpointer data, GClosure * closure)
 {
-    TRACE ("entering closure_notify");
+    TRACE ("entering");
+    
     if (data)
     {
         TRACE ("freeing data");
@@ -320,11 +335,12 @@ closure_notify (gpointer data, GClosure * closure)
 GtkWidget *
 menu_item_connect (GtkWidget * item, MenuData * item_data)
 {
-    TRACE ("entering menu_item_connect");
     g_return_val_if_fail (item != NULL, NULL);
     g_return_val_if_fail (GTK_IS_MENU_ITEM (item), NULL);
-    g_signal_connect_closure (GTK_OBJECT (item), "activate",
-        g_cclosure_new (GTK_SIGNAL_FUNC (activate_cb), item_data,
+    TRACE ("entering");
+
+    g_signal_connect_closure (G_OBJECT (item), "activate",
+        g_cclosure_new (G_CALLBACK (activate_cb), item_data,
             (GClosureNotify) closure_notify), FALSE);
     return (item);
 }
@@ -332,40 +348,90 @@ menu_item_connect (GtkWidget * item, MenuData * item_data)
 gboolean
 menu_is_opened (void)
 {
-    TRACE ("entering menu_is_opened");
+    TRACE ("entering");
     return (menu_open != NULL);
 }
 
 gboolean
 menu_check_and_close (void)
 {
-    TRACE ("entering menu_check_or_close");
+    TRACE ("entering");
     if (menu_open)
     {
-        TRACE ("menu open, emitting deactivate signal");
-        g_signal_emit_by_name (GTK_OBJECT (menu_open), "deactivate");
+        TRACE ("emitting deactivate signal");
+        g_signal_emit_by_name (G_OBJECT (menu_open), "deactivate");
         menu_open = NULL;
         return (TRUE);
     }
     return (FALSE);
 }
 
+static GdkDevice *
+menu_get_device (GdkWindow *win, gint grab_device_type)
+{
+    GdkDisplay *display;
+    GdkSeat *seat;
+
+    display = gdk_window_get_display (win);
+    seat = gdk_display_get_default_seat (display);
+
+    switch (grab_device_type)
+    {
+        case GRAB_DEVICE_POINTER:
+            return gdk_seat_get_pointer (seat);
+        case GRAB_DEVICE_KEYBOARD:
+            return gdk_seat_get_keyboard (seat);
+        default:
+            return NULL;
+    }
+}
+
+static GdkGrabStatus
+menu_device_grab (GdkWindow *win, gint grab_device_type, GdkEventMask event_mask, guint32 timestamp)
+{
+    GdkDevice *device;
+
+    device = menu_get_device (win, grab_device_type);
+
+    g_return_val_if_fail (device != NULL, GDK_GRAB_FAILED);
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+    return gdk_device_grab (device, win, GDK_OWNERSHIP_NONE, TRUE, event_mask, NULL, timestamp);
+G_GNUC_END_IGNORE_DEPRECATIONS
+}
+
+static void
+menu_device_ungrab (GdkWindow *win, gint grab_device_type, guint32 timestamp)
+{
+    GdkDevice *device;
+
+    device = menu_get_device (win, grab_device_type);
+
+    g_return_if_fail (device != NULL);
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+    gdk_device_ungrab (device, timestamp);
+G_GNUC_END_IGNORE_DEPRECATIONS
+}
+
 static gboolean
 grab_available (GdkWindow *win, guint32 timestamp)
 {
-    GdkEventMask mask;
+    GdkEventMask pointer_mask;
+    GdkEventMask keyboard_mask;
     GdkGrabStatus g1;
     GdkGrabStatus g2;
     gboolean grab_failed;
     gint i;
 
-    TRACE ("entering grab_available");
+    TRACE ("entering");
 
-    mask = GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-           GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
-           GDK_POINTER_MOTION_MASK;
-    g1 = gdk_pointer_grab (win, TRUE, mask, NULL, NULL, timestamp);
-    g2 = gdk_keyboard_grab (win, TRUE, timestamp);
+    pointer_mask = GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+                   GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
+                   GDK_POINTER_MOTION_MASK;
+    keyboard_mask = GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK;
+    g1 = menu_device_grab (win, GRAB_DEVICE_POINTER, pointer_mask, timestamp);
+    g2 = menu_device_grab (win, GRAB_DEVICE_KEYBOARD, keyboard_mask, timestamp);
     grab_failed = FALSE;
 
     i = 0;
@@ -376,58 +442,114 @@ grab_available (GdkWindow *win, guint32 timestamp)
         g_usleep (100);
         if (g1 != GDK_GRAB_SUCCESS)
         {
-            g1 = gdk_pointer_grab (win, TRUE, mask, NULL, NULL, timestamp);
+            g1 = menu_device_grab (win, GRAB_DEVICE_POINTER, pointer_mask, timestamp);
         }
         if (g2 != GDK_GRAB_SUCCESS)
         {
-            g2 = gdk_keyboard_grab (win, TRUE, timestamp);
+            g2 = menu_device_grab (win, GRAB_DEVICE_KEYBOARD, keyboard_mask, timestamp);
         }
     }
 
     if (g1 == GDK_GRAB_SUCCESS)
     {
-        gdk_pointer_ungrab (timestamp);
+        menu_device_ungrab (win, GRAB_DEVICE_POINTER, timestamp);
     }
     if (g2 == GDK_GRAB_SUCCESS)
     {
-        gdk_keyboard_ungrab (timestamp);
+        menu_device_ungrab (win, GRAB_DEVICE_KEYBOARD, timestamp);
     }
 
     return (!grab_failed);
 }
 
+#if GTK_CHECK_VERSION(3, 22, 0)
+static GdkEvent *
+menu_popup_event (Menu *menu, gint root_x, gint root_y, guint button, guint32 timestamp,
+                  GdkWindow *window)
+{
+    GdkEvent *event;
+    GdkSeat *seat;
+    GdkDevice *device;
+
+    TRACE ("entering");
+
+    event = gtk_get_current_event ();
+
+    if (event != NULL)
+    {
+        event = gdk_event_copy (event);
+    }
+    else
+    {
+        /* Create fake event since menu can be show without any events */
+
+        seat = gdk_display_get_default_seat (gdk_window_get_display (window));
+        device = gdk_seat_get_pointer (seat);
+
+        event = gdk_event_new (GDK_BUTTON_PRESS);
+        event->button.window = g_object_ref (window);
+        event->button.time = timestamp;
+        event->button.x = event->button.x_root = root_x;
+        event->button.y = event->button.y_root = root_y;
+        event->button.button = button;
+        event->button.device = device;
+        gdk_event_set_device (event, device);
+    }
+
+    return event;
+}
+#endif
+
 gboolean
-menu_popup (Menu * menu, int root_x, int root_y, int button,
-    guint32 timestamp)
+menu_popup (Menu *menu, gint root_x, gint root_y, guint button, guint32 timestamp)
 {
     GdkPoint *pt;
-
-    TRACE ("entering menu_popup");
+    GdkWindow *window;
+#if GTK_CHECK_VERSION(3, 22, 0)
+    GdkEvent *event;
+    GdkRectangle rectangle;
+#endif
 
     g_return_val_if_fail (menu != NULL, FALSE);
     g_return_val_if_fail (GTK_IS_MENU (menu->menu), FALSE);
+    TRACE ("entering");
 
     pt = g_new (GdkPoint, 1);
     pt->x = root_x;
     pt->y = root_y;
 
+    window = gdk_screen_get_root_window (menu->screen);
+
     if (!menu_check_and_close ())
     {
-        if (!grab_available (gdk_screen_get_root_window (menu->screen), timestamp))
+        if (!grab_available (window, timestamp))
         {
             g_free (pt);
-            TRACE ("Cannot get grab on pointer/keyboard, cancel.");
+            TRACE ("cannot get grab on pointer/keyboard, cancel.");
             return FALSE;
         }
         TRACE ("opening new menu");
         menu_open = menu->menu;
         eventFilterPush (menu->filter_setup, menu_filter, NULL);
+
+#if GTK_CHECK_VERSION(3, 22, 0)
+        rectangle.x = root_x;
+        rectangle.y = root_y;
+        rectangle.width = 1;
+        rectangle.height = 1;
+
+        event = menu_popup_event (menu, root_x, root_y, button, timestamp, window);
+        gtk_menu_popup_at_rect (GTK_MENU (menu->menu), window, &rectangle,
+                                GDK_GRAVITY_NORTH_WEST, GDK_GRAVITY_NORTH_WEST, event);
+        gdk_event_free (event);
+#else
         gtk_menu_popup (GTK_MENU (menu->menu), NULL, NULL,
             popup_position_func, pt, 0, timestamp);
+#endif
 
-        if (!GTK_MENU_SHELL (GTK_MENU (menu->menu))->have_xgrab)
+        if (g_object_get_data (G_OBJECT (menu->menu), "gtk-menu-transfer-window") == NULL)
         {
-            gdk_beep ();
+            myDisplayBeep (myDisplayGetDefault ());
             g_message (_("%s: GtkMenu failed to grab the pointer\n"), g_get_prgname ());
             gtk_menu_popdown (GTK_MENU (menu->menu));
             menu_open = NULL;
@@ -441,13 +563,11 @@ menu_popup (Menu * menu, int root_x, int root_y, int button,
 void
 menu_free (Menu * menu)
 {
-    TRACE ("entering menu_free");
-
     g_return_if_fail (menu != NULL);
     g_return_if_fail (menu->menu != NULL);
     g_return_if_fail (GTK_IS_MENU (menu->menu));
 
-    TRACE ("freeing menu");
+    TRACE ("entering");
 
     gtk_widget_destroy (menu->menu);
     g_free (menu);
