@@ -33,6 +33,12 @@
 #include <glib.h>
 #include "screen.h"
 
+#if defined (__GNUC__) && __GNUC__ >= 7
+#define FALLTHROUGH __attribute__ ((fallthrough))
+#else
+#define FALLTHROUGH
+#endif
+
 /*
    Just for completion, being a bit pedantic, X seems to be missing
    those so far, so create them for now.
@@ -54,10 +60,10 @@
 #define Button9 9
 #endif
 
-unsigned int             getMouseXY                             (ScreenInfo *,
+guint                    getMouseXY                             (ScreenInfo *,
                                                                  Window,
-                                                                 int *,
-                                                                 int *);
+                                                                 gint *,
+                                                                 gint *);
 GC                       createGC                               (ScreenInfo *,
                                                                  char *,
                                                                  int,
