@@ -29,6 +29,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
+#include <X11/XKBlib.h>
 #include <X11/extensions/shape.h>
 
 #include <glib.h>
@@ -52,13 +53,15 @@
 #include "workspaces.h"
 #include "xsync.h"
 
-#define MOVERESIZE_EVENT_MASK \
-    PointerMotionHintMask | \
+#define MOVERESIZE_POINTER_EVENT_MASK \
     PointerMotionMask | \
     ButtonMotionMask | \
     ButtonReleaseMask | \
     LeaveWindowMask
 
+#define MOVERESIZE_KEYBOARD_EVENT_MASK \
+    KeyPressMask
+
 #define TILE_DISTANCE 10
 #define BORDER_TILE_LENGTH_RELATIVE 5
 #define use_xor_move(screen_info) (screen_info->params->box_move && !compositorIsActive (screen_info))
@@ -82,7 +85,7 @@ struct _MoveResizeData
     unsigned long configure_flags;
     guint cancel_workspace;
     gint mx, my;
-    gint px, py; /* pointer relative position */
+    double pxratio, pyratio; /* pointer relative position ratio */
     gint ox, oy;
     gint ow, oh;
     gint oldw, oldh;
@@ -96,11 +99,11 @@ clientCheckSize (Client * c, int size, int base, int min, int max, int incr, gbo
     int size_return;
 
     g_return_val_if_fail (c != NULL, size);
-    TRACE ("entering clientCheckSize");
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     size_return = size;
 
-    /* Bypass resize increment and max sizes for fullscreen */
+    /* Bypass resize increment and max sizes for fullscreen and maximized */
     if (!FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
         && !(FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
              && (c->screen_info->params->borderless_maximize)))
@@ -146,8 +149,7 @@ int
 clientCheckWidth (Client * c, int w, gboolean source_is_application)
 {
     g_return_val_if_fail (c != NULL, w);
-    TRACE ("entering clientCheckWidth");
-    TRACE ("setting width %i for client \"%s\" (0x%lx)", w, c->name, c->window);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     return clientCheckSize (c, w,
                             c->size->base_width,
@@ -161,8 +163,7 @@ int
 clientCheckHeight (Client * c, int h, gboolean source_is_application)
 {
     g_return_val_if_fail (c != NULL, h);
-    TRACE ("entering clientCheckHeight");
-    TRACE ("setting height %i for client \"%s\" (0x%lx)", h, c->name, c->window);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     return clientCheckSize (c, h,
                             c->size->base_height,
@@ -183,9 +184,9 @@ clientMovePointer (DisplayInfo *display_info, gint dx, gint dy, guint repeat)
 }
 
 static gboolean
-clientKeyPressIsModifier (XEvent *xevent)
+clientKeyPressIsModifier (XfwmEventKey *event)
 {
-    int keysym = XLookupKeysym (&xevent->xkey, 0);
+    int keysym = XkbKeycodeToKeysym (event->meta.x->xany.display, event->keycode, 0, 0);
     return (gboolean) IsModifierKey(keysym);
 }
 
@@ -246,8 +247,8 @@ clientSetHandle(MoveResizeData *passdata, int handle)
     passdata->handle = handle;
     passdata->mx = px;
     passdata->my = py;
-    passdata->px = passdata->mx - frameExtentX (c);
-    passdata->py = passdata->my - frameExtentY (c);
+    passdata->pxratio = (passdata->mx - frameExtentX (c)) / (double) frameExtentWidth (c);
+    passdata->pyratio = (passdata->my - frameExtentY (c)) / (double) frameExtentHeight (c);
     passdata->ox = c->x;
     passdata->oy = c->y;
     passdata->ow = c->width;
@@ -266,7 +267,6 @@ clientConstrainRatio (Client * c, int handle)
 {
 
     g_return_if_fail (c != NULL);
-    TRACE ("entering clientConstrainRatio");
     TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     if (c->size->flags & PAspect)
@@ -342,7 +342,8 @@ clientConstrainRatio (Client * c, int handle)
 static void
 clientDrawOutline (Client * c)
 {
-    TRACE ("entering clientDrawOutline");
+    g_return_if_fail (c != NULL);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     XDrawRectangle (clientGetXDisplay (c), c->screen_info->xroot, c->screen_info->box_gc, frameExtentX (c), frameExtentY (c),
         frameExtentWidth (c) - 1, frameExtentHeight (c) - 1);
@@ -479,8 +480,7 @@ clientSnapPosition (Client * c, int prev_x, int prev_y)
     GdkRectangle rect;
 
     g_return_if_fail (c != NULL);
-    TRACE ("entering clientSnapPosition");
-    TRACE ("Snapping client \"%s\" (0x%lx)", c->name, c->window);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     screen_info = c->screen_info;
     best_delta_x = screen_info->params->snap_width + 1;
@@ -617,7 +617,7 @@ clientSnapPosition (Client * c, int prev_x, int prev_y)
 }
 
 static eventFilterStatus
-clientButtonReleaseFilter (XEvent * xevent, gpointer data)
+clientButtonReleaseFilter (XfwmEvent *event, gpointer data)
 {
     MoveResizeData *passdata = (MoveResizeData *) data;
     ScreenInfo *screen_info;
@@ -626,13 +626,12 @@ clientButtonReleaseFilter (XEvent * xevent, gpointer data)
     c = passdata->c;
     screen_info = c->screen_info;
 
-    TRACE ("entering clientButtonReleaseFilter");
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
-    if ((xevent->type == ButtonRelease &&
-         (passdata->button == AnyButton ||
-          passdata->button == xevent->xbutton.button)) ||
-        (xevent->type == KeyPress &&
-         xevent->xkey.keycode == screen_info->params->keys[KEY_CANCEL].keycode))
+    if ((event->meta.type == XFWM_EVENT_BUTTON && !event->button.pressed &&
+        (passdata->button == AnyButton || passdata->button == event->button.button)) ||
+        (event->meta.type == XFWM_EVENT_KEY && event->key.pressed &&
+         event->key.keycode == screen_info->params->keys[KEY_CANCEL].keycode))
     {
         gtk_main_quit ();
         return EVENT_FILTER_STOP;
@@ -656,7 +655,7 @@ clientMoveWarp (Client * c, ScreenInfo * screen_info, int * x_root, int * y_root
     g_return_if_fail (screen_info != NULL);
     g_return_if_fail (x_root != NULL);
     g_return_if_fail (y_root != NULL);
-    TRACE ("entering clientMoveWarp");
+    TRACE ("entering");
 
     if ((c != NULL) && !(screen_info->params->wrap_windows))
     {
@@ -783,7 +782,7 @@ clientMoveWarp (Client * c, ScreenInfo * screen_info, int * x_root, int * y_root
 }
 
 static gboolean
-clientMoveTile (Client *c, XMotionEvent *xevent)
+clientMoveTile (Client *c, XfwmEventMotion *event)
 {
     ScreenInfo *screen_info;
     GdkRectangle rect;
@@ -796,8 +795,8 @@ clientMoveTile (Client *c, XMotionEvent *xevent)
         return FALSE;
     }
 
-    x = xevent->x;
-    y = xevent->y;
+    x = event->x;
+    y = event->y;
 
     myScreenFindMonitorAtPoint (screen_info, x, y, &rect);
     disp_x = rect.x;
@@ -814,7 +813,7 @@ clientMoveTile (Client *c, XMotionEvent *xevent)
     {
         /* tile window depending on the mouse position on the screen */
 
-        if ((y > disp_y + dist_corner) && (y < disp_max_y - dist_corner))
+        if ((y >= disp_y + dist_corner) && (y < disp_max_y - dist_corner))
         {
             /* mouse pointer on left edge excluding corners */
             if (x < disp_x + dist)
@@ -833,7 +832,7 @@ clientMoveTile (Client *c, XMotionEvent *xevent)
             /* mouse pointer on top edge excluding corners */
             if (y < disp_y + dist)
             {
-                return clientToggleMaximized (c, CLIENT_FLAG_MAXIMIZED, FALSE);
+                return clientToggleMaximizedAtPoint (c, x, y, CLIENT_FLAG_MAXIMIZED, FALSE);
             }
         }
 
@@ -867,7 +866,7 @@ clientMoveTile (Client *c, XMotionEvent *xevent)
 }
 
 static eventFilterStatus
-clientMoveEventFilter (XEvent * xevent, gpointer data)
+clientMoveEventFilter (XfwmEvent *event, gpointer data)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -880,7 +879,7 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
     unsigned long cancel_maximize_flags;
     unsigned long cancel_restore_size_flags;
 
-    TRACE ("entering clientMoveEventFilter");
+    TRACE ("entering");
 
     c = passdata->c;
     prev_x=c->x;
@@ -895,16 +894,17 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
     moving = FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MOVING_RESIZING);
 
     /* Update the display time */
-    myDisplayUpdateCurrentTime (display_info, xevent);
+    myDisplayUpdateCurrentTime (display_info, event);
 
-    if (xevent->type == KeyPress)
+    if (event->meta.type == XFWM_EVENT_KEY && event->key.pressed)
     {
         int key_move;
 
-        while (XCheckMaskEvent (display_info->dpy, KeyPressMask, xevent))
+        while (xfwm_device_check_mask_event (display_info->devices, display_info->dpy,
+                                             KeyPressMask, event))
         {
             /* Update the display time */
-            myDisplayUpdateCurrentTime (display_info, xevent);
+            myDisplayUpdateCurrentTime (display_info, event);
         }
 
         key_move = 16;
@@ -912,23 +912,23 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
         {
             key_move = MAX (key_move, screen_info->params->snap_width + 1);
         }
-        if (xevent->xkey.keycode == screen_info->params->keys[KEY_LEFT].keycode)
+        if (event->key.keycode == screen_info->params->keys[KEY_LEFT].keycode)
         {
             clientMovePointer (display_info, -1, 0, key_move);
         }
-        else if (xevent->xkey.keycode == screen_info->params->keys[KEY_RIGHT].keycode)
+        else if (event->key.keycode == screen_info->params->keys[KEY_RIGHT].keycode)
         {
             clientMovePointer (display_info, 1, 0, key_move);
         }
-        else if (xevent->xkey.keycode == screen_info->params->keys[KEY_UP].keycode)
+        else if (event->key.keycode == screen_info->params->keys[KEY_UP].keycode)
         {
             clientMovePointer (display_info, 0, -1, key_move);
         }
-        else if (xevent->xkey.keycode == screen_info->params->keys[KEY_DOWN].keycode)
+        else if (event->key.keycode == screen_info->params->keys[KEY_DOWN].keycode)
         {
             clientMovePointer (display_info, 0, 1, key_move);
         }
-        else if (xevent->xkey.keycode == screen_info->params->keys[KEY_CANCEL].keycode)
+        else if (event->key.keycode == screen_info->params->keys[KEY_CANCEL].keycode)
         {
             moving = FALSE;
             passdata->released = passdata->use_keys;
@@ -958,7 +958,7 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
 
             if (screen_info->current_ws != passdata->cancel_workspace)
             {
-                workspaceSwitch (screen_info, passdata->cancel_workspace, c, FALSE, xevent->xkey.time);
+                workspaceSwitch (screen_info, passdata->cancel_workspace, c, FALSE, event->key.time);
             }
             cancel_maximize_flags = passdata->cancel_flags & CLIENT_FLAG_MAXIMIZED;
             if (!FLAG_TEST_AND_NOT(c->flags, cancel_maximize_flags, CLIENT_FLAG_MAXIMIZED))
@@ -990,23 +990,25 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
         }
         else if (passdata->use_keys)
         {
-            moving = clientKeyPressIsModifier(xevent);
+            moving = clientKeyPressIsModifier(&event->key);
         }
     }
-    else if (xevent->type == ButtonRelease)
+    else if (event->meta.type == XFWM_EVENT_BUTTON && !event->button.pressed)
     {
         moving = FALSE;
         passdata->released = (passdata->use_keys ||
                               passdata->button == AnyButton ||
-                              passdata->button == xevent->xbutton.button);
+                              passdata->button == event->button.button);
     }
-    else if (xevent->type == MotionNotify)
+    else if (event->meta.type == XFWM_EVENT_MOTION)
     {
-        while (XCheckMaskEvent (display_info->dpy, PointerMotionMask | ButtonMotionMask, xevent))
+        while (xfwm_device_check_mask_event (display_info->devices, display_info->dpy,
+                                             PointerMotionMask | ButtonMotionMask, event))
         {
             /* Update the display time */
-            myDisplayUpdateCurrentTime (display_info, xevent);
+            myDisplayUpdateCurrentTime (display_info, event);
         }
+
         if (!passdata->grab && use_xor_move(screen_info))
         {
             myDisplayGrabServer (display_info);
@@ -1020,62 +1022,39 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
         if ((screen_info->workspace_count > 1) && !(passdata->is_transient))
         {
             clientMoveWarp (c, screen_info,
-                            &xevent->xmotion.x_root,
-                            &xevent->xmotion.y_root,
-                            xevent->xmotion.time);
+                            &event->motion.x_root,
+                            &event->motion.y_root,
+                            event->motion.time);
         }
 
         if (FLAG_TEST (c->flags, CLIENT_FLAG_RESTORE_SIZE_POS))
         {
+            gboolean size_changed;
 
-            if ((ABS (xevent->xmotion.x_root - passdata->mx) > 15) ||
-                (ABS (xevent->xmotion.y_root - passdata->my) > 15))
+            size_changed = clientToggleMaximized (c, c->flags & CLIENT_FLAG_MAXIMIZED, FALSE);
+            if (clientRestoreSizePos (c))
             {
-                gboolean size_changed;
-                /* to keep the distance from the edges of the window proportional. */
-                double xratio, yratio;
-
-                xratio = (xevent->xmotion.x_root - frameExtentX (c)) / (double) frameExtentWidth (c);
-                yratio = (xevent->xmotion.y_root - frameExtentY (c)) / (double) frameExtentHeight (c);
-
-                size_changed = clientToggleMaximized (c, c->flags & CLIENT_FLAG_MAXIMIZED, FALSE);
-                if (clientRestoreSizePos (c))
-                {
-                    size_changed = TRUE;
-                }
-                if (size_changed)
-                {
-                    passdata->move_resized = TRUE;
-
-                    passdata->ox = c->x;
-                    passdata->mx =  frameExtentX (c) + passdata->px;
-                    if ((passdata->mx <  frameExtentX (c)) || (passdata->mx >  frameExtentX (c) + frameExtentWidth (c)))
-                    {
-                        passdata->mx = CLAMP(frameExtentX (c) + frameExtentWidth (c) * xratio, frameExtentX (c), frameExtentX (c) + frameExtentWidth (c));
-                    }
-
-                    passdata->oy = c->y;
-                    passdata->my = frameExtentY (c) + passdata->py;
-                    if ((passdata->my < frameExtentY (c)) || (passdata->my > frameExtentY (c) + frameExtentHeight (c)))
-                    {
-                        passdata->my = CLAMP(frameExtentY (c) + frameExtentHeight (c) * yratio, frameExtentY (c), frameExtentY (c) + frameExtentHeight (c));
-                    }
-
-                    passdata->configure_flags = CFG_FORCE_REDRAW;
-                }
+                size_changed = TRUE;
             }
-            else
+            if (size_changed)
             {
-                xevent->xmotion.x_root = c->x - passdata->ox + passdata->mx;
-                xevent->xmotion.y_root = c->y - passdata->oy + passdata->my;
+                passdata->move_resized = TRUE;
+
+                /* to keep the distance from the edges of the window proportional. */
+                passdata->ox = c->x;
+                passdata->mx = frameExtentX (c) + passdata->pxratio * frameExtentWidth (c);
+                passdata->oy = c->y;
+                passdata->my = frameExtentY (c) + passdata->pyratio * frameExtentHeight (c);
+
+                passdata->configure_flags = CFG_FORCE_REDRAW;
             }
         }
 
-        c->x = passdata->ox + (xevent->xmotion.x_root - passdata->mx);
-        c->y = passdata->oy + (xevent->xmotion.y_root - passdata->my);
+        c->x = passdata->ox + (event->motion.x_root - passdata->mx);
+        c->y = passdata->oy + (event->motion.y_root - passdata->my);
 
         clientSnapPosition (c, prev_x, prev_y);
-        if (clientMoveTile (c, (XMotionEvent *) xevent))
+        if (clientMoveTile (c, &event->motion))
         {
             passdata->configure_flags = CFG_FORCE_REDRAW;
             passdata->move_resized = TRUE;
@@ -1121,7 +1100,7 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
             passdata->configure_flags = NO_CFG_FLAG;
         }
     }
-    else if ((xevent->type == UnmapNotify) && (xevent->xunmap.window == c->window))
+    else if ((event->meta.x->type == UnmapNotify) && (event->meta.window == c->window))
     {
         moving = FALSE;
         status = EVENT_FILTER_CONTINUE;
@@ -1131,7 +1110,7 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
             clientDrawOutline (c);
         }
     }
-    else if (xevent->type == EnterNotify)
+    else if (event->meta.type == XFWM_EVENT_CROSSING && event->crossing.enter)
     {
         /* Ignore enter events */
     }
@@ -1140,7 +1119,7 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
         status = EVENT_FILTER_CONTINUE;
     }
 
-    TRACE ("leaving clientMoveEventFilter");
+    TRACE ("leaving");
 
     if (!moving)
     {
@@ -1153,7 +1132,7 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
 }
 
 void
-clientMove (Client * c, XEvent * ev)
+clientMove (Client * c, XfwmEventButton *event)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -1163,7 +1142,7 @@ clientMove (Client * c, XEvent * ev)
     gboolean g1, g2;
 
     g_return_if_fail (c != NULL);
-    TRACE ("entering clientDoMove");
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MOVING_RESIZING) ||
         !FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE))
@@ -1176,7 +1155,7 @@ clientMove (Client * c, XEvent * ev)
         return;
     }
 
-    TRACE ("moving client \"%s\" (0x%lx)", c->name, c->window);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
 
@@ -1201,13 +1180,13 @@ clientMove (Client * c, XEvent * ev)
 
     clientSaveSizePos (c);
 
-    if (ev && (ev->type == ButtonPress))
+    if (event && event->pressed)
     {
-        passdata.button = ev->xbutton.button;
-        passdata.mx = ev->xbutton.x_root;
-        passdata.my = ev->xbutton.y_root;
-        passdata.px = passdata.mx - frameExtentX (c);
-        passdata.py = passdata.my - frameExtentY (c);
+        passdata.button = event->button;
+        passdata.mx = event->x_root;
+        passdata.my = event->y_root;
+        passdata.pxratio = (passdata.mx - frameExtentX (c)) / (double) frameExtentWidth (c);
+        passdata.pyratio = (passdata.my - frameExtentY (c)) / (double) frameExtentHeight (c);
     }
     else
     {
@@ -1215,17 +1194,16 @@ clientMove (Client * c, XEvent * ev)
         passdata.released = passdata.use_keys = TRUE;
     }
 
-    g1 = myScreenGrabKeyboard (screen_info, myDisplayGetCurrentTime (display_info));
-    g2 = myScreenGrabPointer (screen_info,
-                              FALSE,
-                              MOVERESIZE_EVENT_MASK,
+    g1 = myScreenGrabKeyboard (screen_info, MOVERESIZE_KEYBOARD_EVENT_MASK,
+                               myDisplayGetCurrentTime (display_info));
+    g2 = myScreenGrabPointer (screen_info, FALSE, MOVERESIZE_POINTER_EVENT_MASK,
                               myDisplayGetCursorMove (display_info),
                               myDisplayGetCurrentTime (display_info));
     if (!g1 || !g2)
     {
         TRACE ("grab failed in clientMove");
 
-        gdk_beep ();
+        myDisplayBeep (display_info);
         myScreenUngrabKeyboard (screen_info, myDisplayGetCurrentTime (display_info));
         myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
 
@@ -1333,7 +1311,7 @@ move_cleanup:
 }
 
 static gboolean
-clientChangeHandle(MoveResizeData *passdata, int handle)
+clientChangeHandle (MoveResizeData *passdata, XfwmEvent *event, int handle)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -1354,7 +1332,7 @@ clientChangeHandle(MoveResizeData *passdata, int handle)
     {
         cursor = myDisplayGetCursorMove (display_info);
     }
-    grab = myScreenChangeGrabPointer (screen_info, MOVERESIZE_EVENT_MASK,
+    grab = myScreenChangeGrabPointer (screen_info, FALSE, MOVERESIZE_POINTER_EVENT_MASK,
                                       cursor, myDisplayGetCurrentTime (display_info));
 
     return (grab);
@@ -1391,7 +1369,7 @@ clientResizeConfigure (Client *c, int pw, int ph)
 }
 
 static eventFilterStatus
-clientResizeEventFilter (XEvent * xevent, gpointer data)
+clientResizeEventFilter (XfwmEvent *event, gpointer data)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -1406,7 +1384,7 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
     int right_edge; /* -Cliff */
     int bottom_edge; /* -Cliff */
 
-    TRACE ("entering clientResizeEventFilter");
+    TRACE ("entering");
 
     passdata = (MoveResizeData *) data;
     c = passdata->c;
@@ -1447,16 +1425,17 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
     prev_height = c->height;
 
     /* Update the display time */
-    myDisplayUpdateCurrentTime (display_info, xevent);
+    myDisplayUpdateCurrentTime (display_info, event);
 
-    if (xevent->type == KeyPress)
+    if (event->meta.type == XFWM_EVENT_KEY && event->key.pressed)
     {
         int key_width_inc, key_height_inc;
 
-        while (XCheckMaskEvent (display_info->dpy, KeyPressMask, xevent))
+        while (xfwm_device_check_mask_event (display_info->devices, display_info->dpy,
+                                             KeyPressMask, event))
         {
             /* Update the display time */
-            myDisplayUpdateCurrentTime (display_info, xevent);
+            myDisplayUpdateCurrentTime (display_info, event);
         }
 
         key_width_inc = c->size->width_inc;
@@ -1471,7 +1450,7 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
             key_height_inc = ((int) (10 / key_height_inc)) * key_height_inc;
         }
 
-        if (xevent->xkey.keycode == screen_info->params->keys[KEY_UP].keycode)
+        if (event->key.keycode == screen_info->params->keys[KEY_UP].keycode)
         {
             if ((passdata->handle == CORNER_COUNT + SIDE_BOTTOM) ||
                 (passdata->handle == CORNER_COUNT + SIDE_TOP))
@@ -1480,10 +1459,10 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
             }
             else
             {
-                clientChangeHandle (passdata, CORNER_COUNT + SIDE_TOP);
+                clientChangeHandle (passdata, event, CORNER_COUNT + SIDE_TOP);
             }
         }
-        else if (xevent->xkey.keycode == screen_info->params->keys[KEY_DOWN].keycode)
+        else if (event->key.keycode == screen_info->params->keys[KEY_DOWN].keycode)
         {
             if ((passdata->handle == CORNER_COUNT + SIDE_BOTTOM) ||
                 (passdata->handle == CORNER_COUNT + SIDE_TOP))
@@ -1492,10 +1471,10 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
             }
             else
             {
-                clientChangeHandle (passdata, CORNER_COUNT + SIDE_BOTTOM);
+                clientChangeHandle (passdata, event, CORNER_COUNT + SIDE_BOTTOM);
             }
         }
-        else if (xevent->xkey.keycode == screen_info->params->keys[KEY_LEFT].keycode)
+        else if (event->key.keycode == screen_info->params->keys[KEY_LEFT].keycode)
         {
             if ((passdata->handle == CORNER_COUNT + SIDE_LEFT) ||
                 (passdata->handle == CORNER_COUNT + SIDE_RIGHT))
@@ -1504,10 +1483,10 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
             }
             else
             {
-                clientChangeHandle (passdata, CORNER_COUNT + SIDE_LEFT);
+                clientChangeHandle (passdata, event, CORNER_COUNT + SIDE_LEFT);
             }
         }
-        else if (xevent->xkey.keycode == screen_info->params->keys[KEY_RIGHT].keycode)
+        else if (event->key.keycode == screen_info->params->keys[KEY_RIGHT].keycode)
         {
             if ((passdata->handle == CORNER_COUNT + SIDE_LEFT) ||
                 (passdata->handle == CORNER_COUNT + SIDE_RIGHT))
@@ -1516,10 +1495,10 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
             }
             else
             {
-                clientChangeHandle (passdata, CORNER_COUNT + SIDE_RIGHT);
+                clientChangeHandle (passdata, event, CORNER_COUNT + SIDE_RIGHT);
             }
         }
-        else if (xevent->xkey.keycode == screen_info->params->keys[KEY_CANCEL].keycode)
+        else if (event->key.keycode == screen_info->params->keys[KEY_CANCEL].keycode)
         {
             resizing = FALSE;
             passdata->released = passdata->use_keys;
@@ -1552,18 +1531,19 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
         }
         else if (passdata->use_keys)
         {
-            resizing = clientKeyPressIsModifier(xevent);
+            resizing = clientKeyPressIsModifier(&event->key);
         }
     }
-    else if (xevent->type == MotionNotify)
+    else if (event->meta.type == XFWM_EVENT_MOTION)
     {
-        while (XCheckMaskEvent (display_info->dpy, ButtonMotionMask | PointerMotionMask, xevent))
+        while (xfwm_device_check_mask_event (display_info->devices, display_info->dpy,
+                                             ButtonMotionMask | PointerMotionMask, event))
         {
             /* Update the display time */
-            myDisplayUpdateCurrentTime (display_info, xevent);
+            myDisplayUpdateCurrentTime (display_info, event);
         }
 
-        if (xevent->type == ButtonRelease)
+        if (event->meta.x->type == ButtonRelease)
         {
             resizing = FALSE;
         }
@@ -1584,7 +1564,7 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
 
         if (move_left)
         {
-            c->width = passdata->ow - (xevent->xmotion.x_root - passdata->mx);
+            c->width = passdata->ow - (event->motion.x_root - passdata->mx);
             c->x = c->x - (c->width - passdata->oldw);
 
             /* Snap the left edge to something. -Cliff */
@@ -1593,7 +1573,7 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
         }
         else if (move_right)
         {
-            c->width = passdata->ow + (xevent->xmotion.x_root - passdata->mx);
+            c->width = passdata->ow + (event->motion.x_root - passdata->mx);
 
             /* Attempt to snap the right edge to something. -Cliff */
             c->width = clientFindClosestEdgeX (c, c->x + c->width + frameExtentRight (c)) - c->x - frameExtentRight (c);
@@ -1603,7 +1583,7 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
         {
             if (move_top)
             {
-                c->height = passdata->oh - (xevent->xmotion.y_root - passdata->my);
+                c->height = passdata->oh - (event->motion.y_root - passdata->my);
                 c->y = c->y - (c->height - passdata->oldh);
 
                 /* Snap the top edge to something. -Cliff */
@@ -1612,7 +1592,7 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
             }
             else if (move_bottom)
             {
-                c->height = passdata->oh + (xevent->xmotion.y_root - passdata->my);
+                c->height = passdata->oh + (event->motion.y_root - passdata->my);
 
                 /* Attempt to snap the bottom edge to something. -Cliff */
                 c->height = clientFindClosestEdgeY (c, c->y + c->height + frameExtentBottom (c)) - c->y - frameExtentBottom (c);
@@ -1661,14 +1641,14 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
             clientResizeConfigure (c, prev_width, prev_height);
         }
     }
-    else if (xevent->type == ButtonRelease)
+    else if (event->meta.type == XFWM_EVENT_BUTTON && !event->button.pressed)
     {
         resizing = FALSE;
         passdata->released = (passdata->use_keys ||
                               passdata->button == AnyButton ||
-                              passdata->button == xevent->xbutton.button);
+                              passdata->button == event->button.button);
     }
-    else if ((xevent->type == UnmapNotify) && (xevent->xunmap.window == c->window))
+    else if ((event->meta.x->type == UnmapNotify) && (event->meta.window == c->window))
     {
         resizing = FALSE;
         status = EVENT_FILTER_CONTINUE;
@@ -1678,7 +1658,7 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
             clientDrawOutline (c);
         }
     }
-    else if (xevent->type == EnterNotify)
+    else if (event->meta.type == XFWM_EVENT_CROSSING && event->crossing.enter)
     {
         /* Ignore enter events */
     }
@@ -1687,7 +1667,7 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
         status = EVENT_FILTER_CONTINUE;
     }
 
-    TRACE ("leaving clientResizeEventFilter");
+    TRACE ("leaving");
 
     if (!resizing)
     {
@@ -1699,7 +1679,7 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
 }
 
 void
-clientResize (Client * c, int handle, XEvent * ev)
+clientResize (Client * c, int handle, XfwmEventButton *event)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -1709,7 +1689,7 @@ clientResize (Client * c, int handle, XEvent * ev)
     gboolean g1, g2;
 
     g_return_if_fail (c != NULL);
-    TRACE ("entering clientResize");
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MOVING_RESIZING))
     {
@@ -1720,7 +1700,7 @@ clientResize (Client * c, int handle, XEvent * ev)
     {
         if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_MOVE))
         {
-            clientMove (c, ev);
+            clientMove (c, event);
         }
         return;
     }
@@ -1733,19 +1713,14 @@ clientResize (Client * c, int handle, XEvent * ev)
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
 
-    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
-        && (screen_info->params->borderless_maximize))
-    {
-        return;
-    }
-
-    TRACE ("resizing client \"%s\" (0x%lx)", c->name, c->window);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     passdata.c = c;
     passdata.cancel_x = passdata.ox = c->x;
     passdata.cancel_y = passdata.oy = c->y;
     passdata.cancel_w = passdata.ow = c->width;
     passdata.cancel_h = passdata.oh = c->height;
+    passdata.configure_flags = NO_CFG_FLAG;
     passdata.use_keys = FALSE;
     passdata.grab = FALSE;
     passdata.released = FALSE;
@@ -1756,11 +1731,11 @@ clientResize (Client * c, int handle, XEvent * ev)
     w_orig = c->width;
     h_orig = c->height;
 
-    if (ev && (ev->type == ButtonPress))
+    if (event && event->pressed)
     {
-        passdata.button = ev->xbutton.button;
-        passdata.mx = ev->xbutton.x_root;
-        passdata.my = ev->xbutton.y_root;
+        passdata.button = event->button;
+        passdata.mx = event->x_root;
+        passdata.my = event->y_root;
     }
     else
     {
@@ -1776,15 +1751,16 @@ clientResize (Client * c, int handle, XEvent * ev)
         cursor = myDisplayGetCursorMove (display_info);
     }
 
-    g1 = myScreenGrabKeyboard (screen_info, myDisplayGetCurrentTime (display_info));
-    g2 = myScreenGrabPointer (screen_info, FALSE, MOVERESIZE_EVENT_MASK,
+    g1 = myScreenGrabKeyboard (screen_info, MOVERESIZE_KEYBOARD_EVENT_MASK,
+                               myDisplayGetCurrentTime (display_info));
+    g2 = myScreenGrabPointer (screen_info, FALSE, MOVERESIZE_POINTER_EVENT_MASK,
                               cursor, myDisplayGetCurrentTime (display_info));
 
     if (!g1 || !g2)
     {
         TRACE ("grab failed in clientResize");
 
-        gdk_beep ();
+        myDisplayBeep (display_info);
         myScreenUngrabKeyboard (screen_info, myDisplayGetCurrentTime (display_info));
         myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
 
@@ -1842,13 +1818,14 @@ clientResize (Client * c, int handle, XEvent * ev)
         if (FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
         {
             clientRemoveMaximizeFlag (c);
+            passdata.configure_flags = CFG_FORCE_REDRAW;
         }
         if (FLAG_TEST (c->flags, CLIENT_FLAG_RESTORE_SIZE_POS))
         {
             FLAG_UNSET (c->flags, CLIENT_FLAG_RESTORE_SIZE_POS);
         }
     }
-    clientReconfigure (c, NO_CFG_FLAG);
+    clientReconfigure (c, passdata.configure_flags);
 
     if (passdata.button != AnyButton && !passdata.released)
     {
