@@ -27,6 +27,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/shape.h>
+#include <X11/extensions/Xinerama.h>
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -43,6 +44,8 @@
 #include <libsn/sn.h>
 #endif
 
+#include <common/xfwm-common.h>
+
 #include "display.h"
 #include "screen.h"
 #include "misc.h"
@@ -60,7 +63,8 @@ myScreenCheckWMAtom (ScreenInfo *screen_info, Atom atom)
     gchar selection[32];
     Atom wm_sn_atom;
 
-    TRACE ("entering myScreenCheckWMAtom");
+    TRACE ("atom %lu", atom);
+
     g_snprintf (selection, sizeof (selection), "WM_S%d", screen_info->screen);
     wm_sn_atom = XInternAtom (myScreenGetXDisplay (screen_info), selection, FALSE);
 
@@ -84,12 +88,12 @@ myScreenSetWMAtom (ScreenInfo *screen_info, gboolean replace_wm)
     g_return_val_if_fail (screen_info, FALSE);
     g_return_val_if_fail (screen_info->display_info, FALSE);
 
-    TRACE ("entering myScreenReplaceWM");
+    TRACE ("replace %i", replace_wm);
 
     display_info = screen_info->display_info;
     g_snprintf (selection, sizeof (selection), "WM_S%d", screen_info->screen);
     wm_sn_atom = XInternAtom (display_info->dpy, selection, FALSE);
-    display_name = gdk_screen_make_display_name (screen_info->gscr);
+    display_name = xfwm_make_display_name (screen_info->gscr);
     wm_name = gdk_x11_screen_get_window_manager_name (screen_info->gscr);
 
     XSync (display_info->dpy, FALSE);
@@ -104,10 +108,10 @@ myScreenSetWMAtom (ScreenInfo *screen_info, gboolean replace_wm)
 
             return FALSE;
         }
-        gdk_error_trap_push ();
+        myDisplayErrorTrapPush (display_info);
         attrs.event_mask = StructureNotifyMask;
         XChangeWindowAttributes (display_info->dpy, current_wm, CWEventMask, &attrs);
-        if (gdk_error_trap_pop ())
+        if (myDisplayErrorTrapPop (display_info))
         {
             current_wm = None;
         }
@@ -170,7 +174,7 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
 
     g_return_val_if_fail (display_info, NULL);
     g_return_val_if_fail (GDK_IS_SCREEN (gscr), NULL);
-    TRACE ("entering myScreenInit");
+    TRACE ("entering");
 
     screen_info = g_new0 (ScreenInfo, 1);
     screen_info->params = g_new0 (XfwmParams, 1);
@@ -199,15 +203,15 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     g_object_unref (G_OBJECT (layout));
 
     screen_info->xscreen = gdk_x11_screen_get_xscreen (gscr);
-    screen_info->xroot = (Window) GDK_DRAWABLE_XID(gdk_screen_get_root_window (gscr));
-    screen_info->screen = gdk_screen_get_number (gscr);
-    screen_info->cmap = GDK_COLORMAP_XCOLORMAP(gdk_screen_get_system_colormap (gscr));
+    screen_info->xroot = gdk_x11_window_get_xid (gdk_screen_get_root_window (gscr));
+    screen_info->screen = gdk_x11_screen_get_screen_number (gscr);
+    screen_info->cmap = DefaultColormapOfScreen (screen_info->xscreen);
     screen_info->depth = DefaultDepth (display_info->dpy, screen_info->screen);
     screen_info->visual = DefaultVisual (display_info->dpy, screen_info->screen);
     screen_info->shape_win = (Window) None;
     myScreenComputeSize (screen_info);
 
-    screen_info->xfwm4_win = GDK_WINDOW_XWINDOW (screen_info->gtk_win->window);
+    screen_info->xfwm4_win = gdk_x11_window_get_xid (gtk_widget_get_window (screen_info->gtk_win));
     if (!myScreenSetWMAtom (screen_info, replace_wm))
     {
         gtk_widget_destroy (screen_info->gtk_win);
@@ -215,7 +219,7 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
         return NULL;
     }
 
-    event_win = eventFilterAddWin (gscr, event_mask);
+    event_win = eventFilterAddWin (gscr, display_info->devices, event_mask);
     if (!event_win)
     {
         gtk_widget_destroy (screen_info->gtk_win);
@@ -301,17 +305,8 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
 #endif
 
     screen_info->font_height = 0;
+    screen_info->font_desc = NULL;
     screen_info->box_gc = None;
-    screen_info->black_gc = NULL;
-    screen_info->white_gc = NULL;
-    screen_info->title_colors[ACTIVE].gc = NULL;
-    screen_info->title_colors[ACTIVE].allocated = FALSE;
-    screen_info->title_colors[INACTIVE].gc = NULL;
-    screen_info->title_colors[INACTIVE].allocated = FALSE;
-    screen_info->title_shadow_colors[ACTIVE].gc = NULL;
-    screen_info->title_shadow_colors[ACTIVE].allocated = FALSE;
-    screen_info->title_shadow_colors[INACTIVE].gc = NULL;
-    screen_info->title_shadow_colors[INACTIVE].allocated = FALSE;
 
     for (i = 0; i < SIDE_COUNT; i++)
     {
@@ -351,7 +346,7 @@ myScreenClose (ScreenInfo *screen_info)
     DisplayInfo *display_info;
 
     g_return_val_if_fail (screen_info, NULL);
-    TRACE ("entering myScreenClose");
+    TRACE ("entering");
 
     display_info = screen_info->display_info;
 
@@ -406,8 +401,7 @@ myScreenGetXDisplay (ScreenInfo *screen_info)
 
     g_return_val_if_fail (screen_info, NULL);
     g_return_val_if_fail (screen_info->display_info, NULL);
-    TRACE ("entering myScreenGetXDisplay");
-
+    
     display_info = screen_info->display_info;
     return display_info->dpy;
 }
@@ -416,7 +410,6 @@ GtkWidget *
 myScreenGetGtkWidget (ScreenInfo *screen_info)
 {
     g_return_val_if_fail (screen_info, NULL);
-    TRACE ("entering myScreenGetGtkWidget");
 
     return screen_info->gtk_win;
 }
@@ -425,28 +418,27 @@ GdkWindow *
 myScreenGetGdkWindow (ScreenInfo *screen_info)
 {
     g_return_val_if_fail (screen_info, NULL);
-    TRACE ("entering myScreenGetGdkWindow");
 
-    return screen_info->gtk_win->window;
+    return gtk_widget_get_window (screen_info->gtk_win);
 }
 
 gboolean
-myScreenGrabKeyboard (ScreenInfo *screen_info, guint32 timestamp)
+myScreenGrabKeyboard (ScreenInfo *screen_info, guint event_mask, guint32 timestamp)
 {
     gboolean grab;
 
     g_return_val_if_fail (screen_info, FALSE);
 
-    TRACE ("entering myScreenGrabKeyboard");
+    TRACE ("timestamp %u", (unsigned int) timestamp);
 
     grab = TRUE;
     if (screen_info->key_grabs == 0)
     {
-        grab = (XGrabKeyboard (myScreenGetXDisplay (screen_info),
-                               screen_info->xroot,
-                               TRUE,
-                               GrabModeAsync, GrabModeAsync,
-                               (Time) timestamp) == GrabSuccess);
+        grab = xfwm_device_grab (screen_info->display_info->devices,
+                                 &screen_info->display_info->devices->keyboard,
+                                 myScreenGetXDisplay (screen_info), screen_info->xroot,
+                                 TRUE, event_mask, GrabModeAsync, screen_info->xroot,
+                                 None, (Time) timestamp);
     }
     screen_info->key_grabs++;
     TRACE ("global key grabs %i", screen_info->key_grabs);
@@ -455,23 +447,22 @@ myScreenGrabKeyboard (ScreenInfo *screen_info, guint32 timestamp)
 }
 
 gboolean
-myScreenGrabPointer (ScreenInfo *screen_info, gboolean owner_events, unsigned int event_mask, Cursor cursor, guint32 timestamp)
+myScreenGrabPointer (ScreenInfo *screen_info, gboolean owner_events,
+                     guint event_mask, Cursor cursor, guint32 timestamp)
 {
     gboolean grab;
 
     g_return_val_if_fail (screen_info, FALSE);
-    TRACE ("entering myScreenGrabPointer");
+    TRACE ("timestamp %u", (unsigned int) timestamp);
 
     grab = TRUE;
     if (screen_info->pointer_grabs == 0)
     {
-        grab = (XGrabPointer (myScreenGetXDisplay (screen_info),
-                              screen_info->xroot,
-                              owner_events, event_mask,
-                              GrabModeAsync, GrabModeAsync,
-                              screen_info->xroot,
-                              cursor,
-                              (Time) timestamp) == GrabSuccess);
+        grab = xfwm_device_grab (screen_info->display_info->devices,
+                                 &screen_info->display_info->devices->pointer,
+                                 myScreenGetXDisplay (screen_info), screen_info->xroot,
+                                 owner_events, event_mask, GrabModeAsync, screen_info->xroot,
+                                 cursor, (Time) timestamp);
     }
     screen_info->pointer_grabs++;
     TRACE ("global pointer grabs %i", screen_info->pointer_grabs);
@@ -480,18 +471,30 @@ myScreenGrabPointer (ScreenInfo *screen_info, gboolean owner_events, unsigned in
 }
 
 gboolean
-myScreenChangeGrabPointer (ScreenInfo *screen_info, unsigned int event_mask, Cursor cursor, guint32 timestamp)
+myScreenChangeGrabPointer (ScreenInfo *screen_info, gboolean owner_events,
+                           guint event_mask, Cursor cursor, guint32 timestamp)
 {
     gboolean grab;
 
     g_return_val_if_fail (screen_info, FALSE);
-    TRACE ("entering myScreenChangeGrabPointer");
+    TRACE ("timestamp %u", (unsigned int) timestamp);
 
     grab = FALSE;
     if (screen_info->pointer_grabs > 0)
     {
-        grab = (XChangeActivePointerGrab (myScreenGetXDisplay (screen_info),
-                                          event_mask, cursor, (Time) timestamp) == GrabSuccess);
+        if (screen_info->display_info->devices->pointer.xi2_device == None)
+        {
+            grab = (XChangeActivePointerGrab (myScreenGetXDisplay (screen_info),
+                                              event_mask, cursor, (Time) timestamp) == GrabSuccess);
+        }
+        else
+        {
+            grab = xfwm_device_grab (screen_info->display_info->devices,
+                                     &screen_info->display_info->devices->pointer,
+                                     myScreenGetXDisplay (screen_info), screen_info->xroot,
+                                     owner_events, event_mask, GrabModeAsync, screen_info->xroot,
+                                     cursor, (Time) timestamp);
+        }
     }
 
     return grab;
@@ -501,7 +504,7 @@ unsigned int
 myScreenUngrabKeyboard (ScreenInfo *screen_info, guint32 timestamp)
 {
     g_return_val_if_fail (screen_info, 0);
-    TRACE ("entering myScreenUngrabKeyboard");
+    TRACE ("timestamp %u", (unsigned int) timestamp);
 
     screen_info->key_grabs--;
     if (screen_info->key_grabs < 0)
@@ -510,7 +513,9 @@ myScreenUngrabKeyboard (ScreenInfo *screen_info, guint32 timestamp)
     }
     if (screen_info->key_grabs == 0)
     {
-        XUngrabKeyboard (myScreenGetXDisplay (screen_info), (Time) timestamp);
+        xfwm_device_ungrab (screen_info->display_info->devices,
+                            &screen_info->display_info->devices->keyboard,
+                            myScreenGetXDisplay (screen_info), (Time) timestamp);
     }
     TRACE ("global key grabs %i", screen_info->key_grabs);
 
@@ -521,7 +526,7 @@ unsigned int
 myScreenUngrabPointer (ScreenInfo *screen_info, guint32 timestamp)
 {
     g_return_val_if_fail (screen_info, 0);
-    TRACE ("entering myScreenUngrabPointer");
+    TRACE ("timestamp %u", (unsigned int) timestamp);
 
     screen_info->pointer_grabs--;
     if (screen_info->pointer_grabs < 0)
@@ -530,7 +535,9 @@ myScreenUngrabPointer (ScreenInfo *screen_info, guint32 timestamp)
     }
     if (screen_info->pointer_grabs == 0)
     {
-        XUngrabPointer (myScreenGetXDisplay (screen_info), (Time) timestamp);
+        xfwm_device_ungrab (screen_info->display_info->devices,
+                            &screen_info->display_info->devices->pointer,
+                            myScreenGetXDisplay (screen_info), (Time) timestamp);
     }
     TRACE ("global pointer grabs %i", screen_info->pointer_grabs);
 
@@ -543,14 +550,14 @@ myScreenGrabKeys (ScreenInfo *screen_info)
     Display *dpy;
     int i;
 
-    TRACE ("entering myScreenUnrabKeys");
     g_return_if_fail (screen_info != NULL);
 
     dpy = myScreenGetXDisplay (screen_info);
 
     for (i = FIRST_KEY; i < KEY_COUNT; i++)
     {
-        grabKey (dpy, &screen_info->params->keys[i], screen_info->xroot);
+        grabKey (screen_info->display_info->devices, dpy,
+                 &screen_info->params->keys[i], screen_info->xroot);
     }
 }
 
@@ -559,24 +566,22 @@ myScreenUngrabKeys (ScreenInfo *screen_info)
 {
     Display *dpy;
 
-    TRACE ("entering myScreenUnrabKeys");
     g_return_if_fail (screen_info != NULL);
 
     dpy = myScreenGetXDisplay (screen_info);
-    ungrabKeys (dpy, screen_info->xroot);
+    ungrabKeys (screen_info->display_info->devices, dpy, screen_info->xroot);
 }
 
-int
-myScreenGetKeyPressed (ScreenInfo *screen_info, XKeyEvent * ev)
+gint
+myScreenGetKeyPressed (ScreenInfo *screen_info, XfwmEventKey *event)
 {
-    int key, state;
+    gint key;
+    guint state;
 
-    TRACE ("entering myScreenGetKeyPressed");
-
-    state = ev->state & MODIFIER_MASK;
+    state = event->state & MODIFIER_MASK;
     for (key = 0; key < KEY_COUNT; key++)
     {
-        if ((screen_info->params->keys[key].keycode == ev->keycode)
+        if ((screen_info->params->keys[key].keycode == event->keycode)
             && (screen_info->params->keys[key].modifier == state))
         {
             break;
@@ -601,7 +606,6 @@ myScreenGetClientFromWindow (ScreenInfo *screen_info, Window w, unsigned short m
     guint i;
 
     g_return_val_if_fail (w != None, NULL);
-    TRACE ("entering myScreenGetClientFromWindow");
     TRACE ("looking for (0x%lx)", w);
 
     for (c = screen_info->clients, i = 0; i < screen_info->client_count; c = c->next, i++)
@@ -626,11 +630,10 @@ myScreenComputeSize (ScreenInfo *screen_info)
 
     g_return_val_if_fail (screen_info != NULL, FALSE);
     g_return_val_if_fail (GDK_IS_SCREEN (screen_info->gscr), FALSE);
-    TRACE ("entering myScreenComputeSize");
 
     width = 0;
     height = 0;
-    num_monitors = gdk_screen_get_n_monitors (screen_info->gscr);
+    num_monitors = xfwm_get_n_monitors (screen_info->gscr);
 
     if (num_monitors == 0)
     {
@@ -639,28 +642,26 @@ myScreenComputeSize (ScreenInfo *screen_info)
 
     for (i = 0; i < num_monitors; i++)
     {
-        gdk_screen_get_monitor_geometry (screen_info->gscr, i, &monitor);
+        xfwm_get_monitor_geometry (screen_info->gscr, i, &monitor);
         width = MAX (monitor.x + monitor.width, width);
         height = MAX (monitor.y + monitor.height, height);
     }
 
-    screen_info->logical_width = gdk_screen_get_width (screen_info->gscr);
-    screen_info->logical_height = gdk_screen_get_height (screen_info->gscr);
+    screen_info->logical_width = WidthOfScreen (screen_info->xscreen);
+    screen_info->logical_height = HeightOfScreen (screen_info->xscreen);
     if ((width != screen_info->logical_width) || (height != screen_info->logical_height))
     {
         g_warning ("output size (%dx%d) and logical screen size (%dx%d) do not match",
                    width, height, screen_info->logical_width, screen_info->logical_height);
     }
 
-    /* Keep the smallest size between what we computed and the
-     * reported size for the screen.
-     */
-    if (width == 0 || width > screen_info->logical_width)
+    /* If we failed to compute the size, use whatever xlib reports */
+    if (width == 0)
     {
         width = screen_info->logical_width;
     }
 
-    if (height == 0 || height > screen_info->logical_height)
+    if (height == 0)
     {
         height = screen_info->logical_height;
     }
@@ -668,7 +669,7 @@ myScreenComputeSize (ScreenInfo *screen_info)
     changed = ((screen_info->width != width) | (screen_info->height != height));
     screen_info->width = width;
     screen_info->height = height;
-    TRACE ("myScreenComputeSize(): width=%i, height=%i", width, height);
+    TRACE ("width=%i, height=%i", width, height);
 
     return changed;
 }
@@ -678,7 +679,6 @@ myScreenGetNumMonitors (ScreenInfo *screen_info)
 {
     g_return_val_if_fail (screen_info != NULL, 0);
     g_return_val_if_fail (screen_info->monitors_index != NULL, 0);
-    TRACE ("entering myScreenGetNMonitors");
 
     return (screen_info->monitors_index->len);
 }
@@ -688,7 +688,6 @@ myScreenGetMonitorIndex (ScreenInfo *screen_info, gint idx)
 {
     g_return_val_if_fail (screen_info != NULL, 0);
     g_return_val_if_fail (screen_info->monitors_index != NULL, 0);
-    TRACE ("entering myScreenGetMonitorIndex");
 
     return (g_array_index (screen_info->monitors_index, gint, idx));
 }
@@ -701,7 +700,6 @@ myScreenRebuildMonitorIndex (ScreenInfo *screen_info)
     gboolean cloned;
 
     g_return_val_if_fail (screen_info != NULL, FALSE);
-    TRACE ("entering myScreenRebuildMonitorIndex");
 
     previous_num_monitors = screen_info->num_monitors;
     screen_info->num_monitors = 0;
@@ -716,14 +714,14 @@ myScreenRebuildMonitorIndex (ScreenInfo *screen_info)
      * the bigger ones first (giving preference to taller monitors
      * over wider monitors)
      */
-    num_monitors = gdk_screen_get_n_monitors (screen_info->gscr);
+    num_monitors = xfwm_get_n_monitors (screen_info->gscr);
     for (i = 0; i < num_monitors; i++)
     {
-        gdk_screen_get_monitor_geometry (screen_info->gscr, i, &monitor);
+        xfwm_get_monitor_geometry (screen_info->gscr, i, &monitor);
         cloned = FALSE;
         for (j = 0; j < (gint) screen_info->monitors_index->len; j++)
         {
-            gdk_screen_get_monitor_geometry (screen_info->gscr, j, &previous);
+            xfwm_get_monitor_geometry (screen_info->gscr, j, &previous);
             if ((previous.x == monitor.x) && (previous.y == monitor.y))
             {
                 cloned = TRUE;
@@ -736,8 +734,8 @@ myScreenRebuildMonitorIndex (ScreenInfo *screen_info)
         }
     }
 
-    TRACE ("Physical monitor reported.: %i", num_monitors);
-    TRACE ("Logical views found.......: %i", screen_info->num_monitors);
+    TRACE ("physical monitor reported.: %i", num_monitors);
+    TRACE ("logical views found.......: %i", screen_info->num_monitors);
 
     return (screen_info->num_monitors != previous_num_monitors);
 }
@@ -746,7 +744,7 @@ void
 myScreenInvalidateMonitorCache (ScreenInfo *screen_info)
 {
     g_return_if_fail (screen_info != NULL);
-    TRACE ("entering myScreenInvalidateMonitorCache");
+    TRACE ("entering");
 
     screen_info->cache_monitor.x = -1;
     screen_info->cache_monitor.y = -1;
@@ -768,7 +766,7 @@ myScreenFindMonitorAtPoint (ScreenInfo *screen_info, gint x, gint y, GdkRectangl
     g_return_if_fail (screen_info != NULL);
     g_return_if_fail (rect != NULL);
     g_return_if_fail (GDK_IS_SCREEN (screen_info->gscr));
-    TRACE ("entering myScreenFindMonitorAtPoint");
+    TRACE ("(%i,%i)", x, y);
 
     /* Cache system */
     if ((x >= screen_info->cache_monitor.x) && (x < screen_info->cache_monitor.x + screen_info->cache_monitor.width) &&
@@ -786,7 +784,7 @@ myScreenFindMonitorAtPoint (ScreenInfo *screen_info, gint x, gint y, GdkRectangl
         gint monitor_index;
 
         monitor_index = myScreenGetMonitorIndex (screen_info, i);
-        gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_index, &monitor);
+        xfwm_get_monitor_geometry (screen_info->gscr, monitor_index, &monitor);
 
         if ((x >= monitor.x) && (x < (monitor.x + monitor.width)) &&
             (y >= monitor.y) && (y < (monitor.y + monitor.height)))
@@ -815,6 +813,55 @@ myScreenFindMonitorAtPoint (ScreenInfo *screen_info, gint x, gint y, GdkRectangl
     *rect = screen_info->cache_monitor;
 }
 
+void
+myScreenGetXineramaMonitorGeometry (ScreenInfo *screen_info, gint monitor_num, GdkRectangle *rect)
+{
+    XineramaScreenInfo *infos;
+    int n;
+
+    g_return_if_fail (screen_info != NULL);
+    g_return_if_fail (rect != NULL);
+    g_return_if_fail (XineramaIsActive (myScreenGetXDisplay (screen_info)));
+
+    infos = XineramaQueryScreens (myScreenGetXDisplay (screen_info), &n);
+    if (infos == NULL || n <= 0 || monitor_num > n)
+    {
+        g_warning ("Cannot query Xinerama!");
+        XFree (infos);
+
+        /* Using screen size as fallback, safer than some random values */
+        rect->x = 0;
+        rect->y = 0;
+        rect->width = screen_info->width;
+        rect->height = screen_info->height;
+
+        return;
+    }
+
+    rect->x = infos[monitor_num].x_org;
+    rect->y = infos[monitor_num].y_org;
+    rect->width = infos[monitor_num].width;
+    rect->height = infos[monitor_num].height;
+
+    XFree (infos);
+}
+
+PangoFontDescription *
+myScreenGetFontDescription (ScreenInfo *screen_info)
+{
+    GtkWidget *widget;
+
+    g_return_val_if_fail (screen_info != NULL, FALSE);
+
+    if (screen_info->font_desc != NULL)
+    {
+        return screen_info->font_desc;
+    }
+
+    widget = myScreenGetGtkWidget (screen_info);
+    return getUIPangoFontDesc (widget);
+}
+
 gboolean
 myScreenUpdateFontHeight (ScreenInfo *screen_info)
 {
@@ -824,13 +871,12 @@ myScreenUpdateFontHeight (ScreenInfo *screen_info)
     GtkWidget *widget;
 
     g_return_val_if_fail (screen_info != NULL, FALSE);
-    TRACE ("entering myScreenUpdateFontHeight");
 
     widget = myScreenGetGtkWidget (screen_info);
+    desc = myScreenGetFontDescription (screen_info);
     context = getUIPangoContext (widget);
-    desc = getUIPangoFontDesc (widget);
 
-    if (desc && context)
+    if (desc != NULL && context != NULL)
     {
         metrics = pango_context_get_metrics (context, desc, NULL);
         screen_info->font_height =
@@ -843,3 +889,4 @@ myScreenUpdateFontHeight (ScreenInfo *screen_info)
 
     return FALSE;
 }
+
