@@ -86,8 +86,8 @@ struct _ScreenInfo
     gint pointer_grabs;
 
     /* Theme pixmaps and other params, per screen */
-    XfwmColor title_colors[2];
-    XfwmColor title_shadow_colors[2];
+    GdkRGBA title_colors[2];
+    GdkRGBA title_shadow_colors[2];
     xfwmPixmap buttons[BUTTON_COUNT][STATE_COUNT];
     xfwmPixmap corners[CORNER_COUNT][2];
     xfwmPixmap sides[SIDE_COUNT][2];
@@ -96,11 +96,10 @@ struct _ScreenInfo
 
     /* Per screen graphic contexts */
     GC box_gc;
-    GdkGC *black_gc;
-    GdkGC *white_gc;
 
-    /* Title font height */
+    /* Title font */
     gint font_height;
+    PangoFontDescription *font_desc;
 
     /* Screen data */
     Colormap cmap;
@@ -207,9 +206,6 @@ struct _ScreenInfo
     gboolean use_present;
 
 #ifdef HAVE_EPOXY
-    gboolean has_glx_sync_control;
-    gboolean has_glx_video_sync;
-    gboolean has_texture_from_pixmap;
     gboolean texture_inverted;
 
     GLuint rootTexture;
@@ -241,14 +237,16 @@ Display                 *myScreenGetXDisplay                    (ScreenInfo *);
 GtkWidget               *myScreenGetGtkWidget                   (ScreenInfo *);
 GdkWindow               *myScreenGetGdkWindow                   (ScreenInfo *);
 gboolean                 myScreenGrabKeyboard                   (ScreenInfo *,
+                                                                 guint,
                                                                  guint32);
 gboolean                 myScreenGrabPointer                    (ScreenInfo *,
                                                                  gboolean,
-                                                                 unsigned int,
+                                                                 guint,
                                                                  Cursor,
                                                                  guint32);
 gboolean                 myScreenChangeGrabPointer              (ScreenInfo *,
-                                                                 unsigned int,
+                                                                 gboolean,
+                                                                 guint,
                                                                  Cursor,
                                                                  guint32);
 unsigned int             myScreenUngrabKeyboard                 (ScreenInfo *,
@@ -257,8 +255,8 @@ unsigned int             myScreenUngrabPointer                  (ScreenInfo *,
                                                                  guint32);
 void                     myScreenGrabKeys                       (ScreenInfo *);
 void                     myScreenUngrabKeys                     (ScreenInfo *);
-int                      myScreenGetKeyPressed                  (ScreenInfo *,
-                                                                 XKeyEvent *);
+gint                     myScreenGetKeyPressed                  (ScreenInfo *,
+                                                                 XfwmEventKey *);
 int                      myScreenGetModifierPressed             (ScreenInfo *);
 Client                  *myScreenGetClientFromWindow            (ScreenInfo *,
                                                                  Window,
@@ -273,6 +271,9 @@ void                     myScreenFindMonitorAtPoint             (ScreenInfo *,
                                                                  gint,
                                                                  gint,
                                                                  GdkRectangle *);
+PangoFontDescription *   myScreenGetFontDescription             (ScreenInfo *);
 gboolean                 myScreenUpdateFontHeight               (ScreenInfo *);
-
+void                     myScreenGetXineramaMonitorGeometry     (ScreenInfo *,
+                                                                 gint,
+                                                                 GdkRectangle *);
 #endif /* INC_SCREEN_H */
