@@ -248,50 +248,6 @@ loadXfconfData (ScreenInfo *screen_info, Settings *rc)
 
 }
 
-/* Simple helper function to avoid copy/paste of code */
-static void
-setXfwmColor (ScreenInfo *screen_info, XfwmColor *color, Settings *rc, int id, const gchar * name, const gchar * state)
-{
-    if (color->allocated)
-    {
-        gdk_colormap_free_colors (gdk_screen_get_system_colormap (screen_info->gscr), &color->col, 1);
-        color->allocated = FALSE;
-    }
-
-    /** do a direct value_get_string */
-    if (gdk_color_parse (g_value_get_string(rc[id].value), &color->col))
-    {
-        if (gdk_colormap_alloc_color (gdk_screen_get_system_colormap (screen_info->gscr),
-                                      &color->col, FALSE, FALSE))
-        {
-            color->allocated = TRUE;
-            if (color->gc)
-            {
-                g_object_unref (G_OBJECT (color->gc));
-            }
-            color->gc = gdk_gc_new (myScreenGetGdkWindow (screen_info));
-            gdk_gc_copy (color->gc, getUIStyle_gc (myScreenGetGtkWidget (screen_info), name, state));
-            gdk_gc_set_foreground (color->gc, &color->col);
-        }
-        else
-        {
-            gdk_beep ();
-            if (G_VALUE_TYPE(rc[id].value) == G_TYPE_STRING)
-                g_message (_("%s: Cannot allocate color %s\n"), g_get_prgname (), g_value_get_string(rc[id].value));
-            else
-                g_critical (_("%s: Cannot allocate color: GValue for color is not of type STRING"), g_get_prgname ());
-        }
-    }
-    else
-    {
-        gdk_beep ();
-        if (G_VALUE_TYPE(rc[id].value) == G_TYPE_STRING)
-            g_message (_("%s: Cannot parse color %s\n"), g_get_prgname (), g_value_get_string(rc[id].value));
-        else
-            g_critical (_("%s: Cannot parse color: GValue for color is not of type STRING"), g_get_prgname ());
-    }
-}
-
 static int
 getTitleShadow (Settings *rc, const gchar * name)
 {
@@ -411,13 +367,11 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     GtkWidget *widget;
     gchar *theme;
     const gchar *font;
-    PangoFontDescription *desc;
     guint i, j;
 
     widget = myScreenGetGtkWidget (screen_info);
     display_info = screen_info->display_info;
 
-    desc = NULL;
     i = 0;
 
     /* Load gtk theme colors first */
@@ -425,7 +379,7 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     {
         gchar *color;
 
-        color = getUIStyle  (widget, ui_part[i], ui_state[i]);
+        color = getUIStyleString (widget, ui_part[i], ui_state[i]);
         setStringValue (rc[i].option, color, rc);
         g_free (color);
     }
@@ -495,33 +449,14 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     font = getStringValue ("title_font", rc);
     if (font && strlen (font))
     {
-        desc = pango_font_description_from_string (font);
-        if (desc)
-        {
-            gtk_widget_modify_font (widget, desc);
-            pango_font_description_free (desc);
-            myScreenUpdateFontHeight (screen_info);
-        }
+        screen_info->font_desc = pango_font_description_from_string (font);
     }
+    myScreenUpdateFontHeight (screen_info);
 
-    setXfwmColor (screen_info, &screen_info->title_colors[ACTIVE], rc, 0, "fg", "selected");
-    setXfwmColor (screen_info, &screen_info->title_colors[INACTIVE], rc, 1, "fg", "insensitive");
-    setXfwmColor (screen_info, &screen_info->title_shadow_colors[ACTIVE], rc, 2, "dark", "selected");
-    setXfwmColor (screen_info, &screen_info->title_shadow_colors[INACTIVE], rc, 3, "dark", "insensitive");
-
-    if (screen_info->black_gc)
-    {
-        g_object_unref (G_OBJECT (screen_info->black_gc));
-    }
-    screen_info->black_gc = widget->style->black_gc;
-    g_object_ref (G_OBJECT (widget->style->black_gc));
-
-    if (screen_info->white_gc)
-    {
-        g_object_unref (G_OBJECT (screen_info->white_gc));
-    }
-    screen_info->white_gc = widget->style->white_gc;
-    g_object_ref (G_OBJECT (widget->style->white_gc));
+    gdk_rgba_parse (&screen_info->title_colors[ACTIVE], getStringValue ("active_text_color", rc));
+    gdk_rgba_parse (&screen_info->title_colors[INACTIVE], getStringValue ("inactive_text_color", rc));
+    gdk_rgba_parse (&screen_info->title_shadow_colors[ACTIVE], getStringValue ("active_text_shadow_color", rc));
+    gdk_rgba_parse (&screen_info->title_shadow_colors[INACTIVE], getStringValue ("inactive_text_shadow_color", rc));
 
     for (i = 0; i < SIDE_COUNT; i++)
     {
@@ -719,6 +654,7 @@ loadSettings (ScreenInfo *screen_info)
         {"click_to_focus", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_apps_only", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_draw_frame", NULL, G_TYPE_BOOLEAN, TRUE},
+        {"cycle_raise", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_hidden", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_minimum", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_preview", NULL, G_TYPE_BOOLEAN, TRUE},
@@ -791,7 +727,7 @@ loadSettings (ScreenInfo *screen_info)
         {NULL, NULL, G_TYPE_INVALID, FALSE}
     };
 
-    TRACE ("entering loadSettings");
+    TRACE ("entering");
 
     loadRcData (screen_info, rc);
     loadXfconfData (screen_info, rc);
@@ -816,6 +752,8 @@ loadSettings (ScreenInfo *screen_info)
         getBoolValue ("cycle_minimum", rc);
     screen_info->params->cycle_draw_frame =
         getBoolValue ("cycle_draw_frame", rc);
+    screen_info->params->cycle_raise =
+        getBoolValue ("cycle_raise", rc);
     screen_info->params->cycle_hidden =
         getBoolValue ("cycle_hidden", rc);
     screen_info->params->cycle_preview =
@@ -878,8 +816,6 @@ loadSettings (ScreenInfo *screen_info)
         getBoolValue ("snap_resist", rc);
     screen_info->params->snap_width =
         getIntValue ("snap_width", rc);
-    screen_info->params->sync_to_vblank =
-        getBoolValue ("sync_to_vblank", rc);
     screen_info->params->tile_on_move =
         getBoolValue ("tile_on_move", rc);
     screen_info->params->toggle_workspaces =
@@ -953,7 +889,13 @@ unloadTheme (ScreenInfo *screen_info)
 {
     int i, j;
 
-    TRACE ("entering unloadTheme");
+    TRACE ("entering");
+
+    if (screen_info->font_desc != NULL)
+    {
+        pango_font_description_free (screen_info->font_desc);
+        screen_info->font_desc = NULL;
+    }
 
     for (i = 0; i < SIDE_COUNT; i++)
     {
@@ -1005,7 +947,7 @@ unloadSettings (ScreenInfo *screen_info)
 {
     g_return_if_fail (screen_info);
 
-    TRACE ("entering unloadSettings");
+    TRACE ("entering");
 
     unloadTheme (screen_info);
     unloadKeyBindings (screen_info);
@@ -1040,7 +982,7 @@ reloadSettings (DisplayInfo *display_info, int mask)
 
     g_return_val_if_fail (display_info, FALSE);
 
-    TRACE ("entering reloadSettings");
+    TRACE ("entering");
 
     /* Refresh all screens, not just one */
     for (screens = display_info->screens; screens; screens = g_slist_next (screens))
@@ -1066,7 +1008,7 @@ initSettings (ScreenInfo *screen_info)
 
     g_return_val_if_fail (screen_info, FALSE);
 
-    TRACE ("entering initSettings");
+    TRACE ("entering");
 
     if (!xfconf_init (NULL))
     {
@@ -1083,7 +1025,7 @@ initSettings (ScreenInfo *screen_info)
     g_signal_connect (screen_info->xfwm4_channel, "property-changed",
                       G_CALLBACK (cb_xfwm4_channel_property_changed), screen_info);
 
-    keymap = gdk_keymap_get_default ();
+    keymap = myDisplayGetKeymap (display_info);
     g_signal_connect (keymap, "keys-changed",
                       G_CALLBACK (cb_keys_changed), screen_info);
 
@@ -1341,6 +1283,10 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     screen_info->params->cycle_draw_frame = g_value_get_boolean (value);
                 }
+                else if (!strcmp (name, "cycle_raise"))
+                {
+                    screen_info->params->cycle_raise = g_value_get_boolean (value);
+                }
                 else if (!strcmp (name, "cycle_hidden"))
                 {
                     screen_info->params->cycle_hidden = g_value_get_boolean (value);
@@ -1405,10 +1351,6 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     screen_info->params->tile_on_move = g_value_get_boolean (value);
                 }
-                else if (!strcmp (name, "sync_to_vblank"))
-                {
-                    screen_info->params->sync_to_vblank = g_value_get_boolean (value);
-                }
                 else if (!strcmp (name, "toggle_workspaces"))
                 {
                     screen_info->params->toggle_workspaces = g_value_get_boolean (value);
@@ -1462,7 +1404,7 @@ keymap_reload (gpointer data)
     ScreenInfo *screen_info = (ScreenInfo *) data;
 
     g_return_val_if_fail (screen_info != NULL, FALSE);
-    TRACE ("entering keymap_reload");
+    TRACE ("entering");
 
     /* Recompute modifiers mask in case it changed */
     initModifiers (myScreenGetXDisplay (screen_info));
